#! /usr/bin/env python
#
"""
"""
# Copyright (C) 2011, GC3, University of Zurich. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
__docformat__ = 'reStructuredText'
__version__ = '$Revision$'

def run_my_tests(local_variables):
    errors = 0
    tests = 0
    for i in sorted(local_variables):
        if not i.startswith('test_'): continue
        tests += 1
        try:
            print "%s ... " % i,
            local_variables[i]()
            print "ok"
        except Exception, e:
            print "FAIL"
            print "Error in function %s" % i
            print "Exception %s: %s" % (repr(e), e)
            errors +=1
            print
    print
    print "Run %d tests" % tests
    if errors: print "Errors: %d" % errors
    else: print "No errors! Good!"




#! /usr/bin/env python
#
"""
Generate the same input files structure as `gricomp.py`, but without
running any job.
"""
# Copyright (C) 2011 GC3, University of Zurich. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
__version__ = '$Revision$'
__author__ = 'Riccardo Murri <riccardo.murri@uzh.ch>'
# summary of user-visible changes
__changelog__ = """
  2011-06-20:
    * Copy the ``coord`` file into each generated ``ridft`` and ``ricc2`` directory.
    * Move the ``ricc2`` directory at the leaf of the generated tree.
  2011-06-09:
    * Forked off from the `gricomp.py` source code.
"""
__docformat__ = 'reStructuredText'


import fnmatch
import os
import os.path
import shutil
import sys

## interface to Gc3libs

import gc3libs
from gc3libs.cmdline import GC3UtilsScript, positive_int, valid_directory
from gc3libs.template import Template, expansions
import gc3libs.utils


## support classes

# list basis set names (as accepted by "define") here;
# ORDER IS IMPORTANT: the `acceptable_*_basis_set` functions
# will only allow a jkbas, cbas or cabs if it's not *earlier*
# than the orbital basis; i.e. if the orbital basis is
# ``aug-cc-pVQZ`` then jkbas cannot be ``aug-cc-pVTZ``.
basis_set_names = [
    'aug-cc-pVTZ',
    'aug-cc-pVQZ',
    'aug-cc-pV5Z',
    ]

def acceptable_ridft_basis_set(extra_args):
    """Define which combination of orbital and JK basis are valid."""
    def order(k): # small aux function
        return basis_set_names.index(extra_args[k])
    orb_basis_nr = order('ORB_BASIS')
    jkbas_basis_nr = order('RIJK_BASIS')
    # only allow a jkbas if it's not *earlier* than the orbital basis;
    # i.e. if the orbital basis is ``aug-cc-pVQZ`` then jkbas cannot
    # be ``aug-cc-pVTZ``.
    if orb_basis_nr > jkbas_basis_nr:
        return False
    # otherwise, the basis combination is acceptable
    return True

def acceptable_ricc2_basis_set(extra_args):
    """Define which combination of CABS and CBAS are valid."""
    def order(k): # small aux function
        return basis_set_names.index(extra_args[k])
    orb_basis_nr = order('ORB_BASIS')
    cabs_basis_nr = order('CABS_BASIS')
    cbas_basis_nr = order('CBAS_BASIS')
    # only allow a cbas or cabs if it's not *earlier*
    # than the orbital basis; i.e. if the orbital basis is
    # ``aug-cc-pVQZ`` then cbas cannot be ``aug-cc-pVTZ``.
    if orb_basis_nr > cbas_basis_nr:
        return False
    if orb_basis_nr > cabs_basis_nr:
        return False
    # otherwise, the basis combination is acceptable
    return True

# TURBOMOLE's ``define`` input for the RIDFT step
RIDFT_DEFINE_IN = """\

${TITLE}
a coord
*
no
b all ${ORB_BASIS}
*
eht



scf
conv
8

ri
on

rijk
on
jkbas
b all ${RIJK_BASIS}
*
m ${RIDFT_MEMORY}

*
"""

# TURBOMOLE's ``define`` input for the RICC2 step
RICC2_DEFINE_IN = """\





cc
freeze
*
cbas
b all ${CBAS_BASIS}
*
memory ${RICC2_MEMORY}
denconv 1.0d-8
ricc2
mp2 energy only
*
f12
pairenergy on
*
cabs
b all ${CABS_BASIS}
*
*
*
"""


## main

class CreateTMFilesScript(GC3UtilsScript):
    """
For each molecule defined in a ``coord`` file given on the
command-line, create TURBOMOLE's ``ridft`` and then ``ricc2`` input
files, with each possible combination of orbital and auxiliary basis
sets.

The list of orbital and auxiliary basis sets to try can be
controlled with the ``--bas``, ``--jkbas``, ``--cbas`` and
``--cabs`` options.
    """

    def __init__(self):
        GC3UtilsScript.__init__(
            self,
            version = '0.1',
            # TURBOMOLE's "coord" files are input
            input_filename_pattern = 'coord',
            )

    def setup_args(self):
        """
        Set up command-line parsing.
        """
        # option arguments
        self.add_param("--bas", metavar="LIST", action="append",
                       dest="bas", default=[],
                       help="Comma-separated list of orbital bases to sweep."
                       " (Default: %(default)s")
        self.add_param("--jkbas", metavar="LIST", action="append",
                       dest="jkbas", default=['aug-cc-pVTZ', 'aug-cc-pVQZ', 'aug-cc-pV5Z'],
                       help="Comma-separated list of RIJK bases to sweep."
                       " (Default: %(default)s")
        self.add_param("--cbas", metavar="LIST", action="append",
                       dest="cbas", default=['aug-cc-pVTZ', 'aug-cc-pVQZ', 'aug-cc-pV5Z'],
                       help="Comma-separated list of `cbas` bases to sweep."
                       " (Default: %(default)s")
        self.add_param("--cabs", metavar="LIST", action="append",
                       dest="cabs", default=['aug-cc-pVTZ', 'aug-cc-pVQZ', 'aug-cc-pV5Z'],
                       help="Comma-separated list of `cabs` bases to sweep."
                       " (Default: %(default)s")
        self.add_param("-m", "--memory",
                       dest="memory", type=positive_int, default=2000,
                       help="Memory (in MB) to set for TURBOMOLE jobs.")
        self.add_param("-o", "--output-directory", metavar='PATH',
                       dest="output_dir", type=valid_directory, default=os.getcwd(),
                       help="Create output files into directories rooted at PATH")
        # positional (mandatory) arguments
        self.add_param('args', nargs='+', metavar='COORDFILE',
                       help="Path to a `coord` file in TURBOMOLE format.")

    def parse_args(self):
        # collect the basis set names given to the ``--bas``,
        # ``--jkbas``, ``--cbas`` and ``--cabs`` options and make them
        # into properly formatted lists.

        if len(self.params.bas) == 0:
            self.params.bas = basis_set_names
        else:
            self.params.bas = str.join(',', self.params.bas).split(',')
        for name in self.params.bas:
            if name not in basis_set_names:
                raise gc3libs.exceptions.InvalidUsage(
                    "Unknown basis set name: '%s'." % name)

        if len(self.params.jkbas) == 0:
            self.params.jkbas = jkbasis_set_names
        else:
            self.params.jkbas = str.join(',', self.params.jkbas).split(',')
        for name in self.params.jkbas:
            if name not in basis_set_names:
                raise gc3libs.exceptions.InvalidUsage(
                    "Unknown basis set name: '%s'." % name)

        if len(self.params.cbas) == 0:
            self.params.cbas = cbasis_set_names
        else:
            self.params.cbas = str.join(',', self.params.cbas).split(',')
        for name in self.params.cbas:
            if name not in basis_set_names:
                raise gc3libs.exceptions.InvalidUsage(
                    "Unknown basis set name: '%s'." % name)

        if len(self.params.cabs) == 0:
            self.params.cabs = cabsis_set_names
        else:
            self.params.cabs = str.join(',', self.params.cabs).split(',')
        for name in self.params.cabs:
            if name not in basis_set_names:
                raise gc3libs.exceptions.InvalidUsage(
                    "Unknown basis set name: '%s'." % name)

        if self.params.memory < 1:
            raise gc3libs.exceptions.InvalidUsage(
                "Argument to option -m/--memory must be a positive integer;"
                " got %d instead" % self.params.memory)


    def _search_for_input_files(self, paths):
        """
        Recursively scan each location in list `paths` for files
        matching the `self.input_filename_pattern` glob pattern, and
        return the set of path names to such files.
        """
        inputs = set()

        pattern = self.input_filename_pattern
        # special case for '*.ext' patterns
        ext = None
        if pattern.startswith('*.'):
            ext = pattern[1:]
            # re-check for more wildcard characters
            if '*' in ext or '?' in ext or '[' in ext:
                ext = None
        #self.log.debug("Input files must match glob pattern '%s' or extension '%s'"
        #               % (pattern, ext))

        def matches(name):
            return (fnmatch.fnmatch(os.path.basename(name), pattern)
                    or fnmatch.fnmatch(name, pattern))
        for path in paths:
            self.log.debug("Now processing input path '%s' ..." % path)
            if os.path.isdir(path):
                # recursively scan for input files
                for dirpath, dirnames, filenames in os.walk(path):
                    for filename in filenames:
                        if matches(filename):
                            self.log.debug("Path '%s' matches pattern '%s',"
                                           " adding it to input list"
                                           % (os.path.join(dirpath, filename),
                                              pattern))
                            inputs.add(os.path.join(dirpath, filename))
            elif matches(path) and os.path.exists(path):
                self.log.debug("Path '%s' matches pattern '%s',"
                               " adding it to input list" % (path, pattern))
                inputs.add(path)
            elif ext is not None and not path.endswith(ext) and os.path.exists(path + ext):
                self.log.debug("Path '%s' matched extension '%s',"
                               " adding to input list"
                               % (path + ext, ext))
                inputs.add(os.path.realpath(path + ext))
            else:
                self.log.error("Cannot access input path '%s' - ignoring it.", path)
            #self.log.debug("Gathered input files: '%s'" % str.join("', '", inputs))

        return inputs


    def _make_define_in(self, path, contents):
        """
        Write `contents` to a file named ``define.in`` in directory
        `path`.  Return the full path to the written file.
        """
        define_in_filename = os.path.join(path, 'define.in')
        define_in_file = open(define_in_filename, 'w')
        define_in_file.write(str(contents))
        define_in_file.close()
        return define_in_filename


    def _make_turbomole_files(self, coord, ridft_in, ricc2_ins, work_dir):
        orb_basis = ridft_in._keywords['ORB_BASIS']
        rijk_basis = ridft_in._keywords['RIJK_BASIS']
        work_dir = os.path.join(work_dir,
                                'bas-%s/jkbas-%s' % (orb_basis, rijk_basis))
        gc3libs.utils.mkdir(work_dir)
        # run 1st pass in the `ridft` directory
        ridft_dir = os.path.join(work_dir, 'ridft')
        gc3libs.utils.mkdir(ridft_dir)
        ridft_coord = os.path.join(ridft_dir, 'coord')
        gc3libs.utils.copyfile(coord, ridft_coord)
        ridft_define_in = self._make_define_in(ridft_dir, ridft_in)
        gc3libs.log.info("Created RIDFT input files in directory '%s'",
                         ridft_dir)
        # proceeed with 2nd pass
        for ricc2_in in ricc2_ins:
            cbas = ricc2_in._keywords['CBAS_BASIS']
            cabs = ricc2_in._keywords['CABS_BASIS']
            ricc2_dir = os.path.join(work_dir,
                                     'cbas-%s/cabs-%s/ricc2' % (cbas, cabs))
            gc3libs.utils.mkdir(ricc2_dir)
            gc3libs.utils.copyfile(ridft_coord, ricc2_dir)
            ricc2_define_in = self._make_define_in(ricc2_dir, ricc2_in)
            gc3libs.log.info("Created RICC2 input files in directory '%s'",
                             ricc2_dir)


    def main(self):
        coords = self._search_for_input_files(self.params.args)
        for coord in coords:
            gc3libs.log.info("Processing input file '%s' ..." % coord)
            # XXX: how do we get a unique name for each coord?  for
            # now, assume the directory containing the `coord` file
            # gives the unique name
            name = os.path.basename(os.path.dirname(coord))

            ridft_define_in = Template(
                RIDFT_DEFINE_IN, acceptable_ridft_basis_set,
                TITLE=name,
                ORB_BASIS=self.params.bas,
                RIJK_BASIS=self.params.jkbas,
                RIDFT_MEMORY = [self.params.memory]
                ) # end of RIDFT template

            ricc2_define_in = Template(
                RICC2_DEFINE_IN, acceptable_ricc2_basis_set,
                # the ORB_BASIS will be derived from the RIDFT_DEFINE_IN template
                CBAS_BASIS=self.params.cbas,
                CABS_BASIS=self.params.cabs,
                RICC2_MEMORY = [self.params.memory],
                ) # end of RICC2 template

            for ridft_in in expansions(ridft_define_in):
                orb_basis = ridft_in._keywords['ORB_BASIS']
                self._make_turbomole_files(
                    coord,
                    ridft_in,
                    # ricc2_ins
                    list(expansions(ricc2_define_in,
                                    ORB_BASIS=orb_basis)),
                    os.path.join(self.params.output_dir, name))



# run script
if __name__ == '__main__':
    CreateTMFilesScript().run()



#!/usr/bin/env python
#
#   gzods.py -- Front-end script for submitting multiple ZODS jobs
#
#   Copyright (C) 2012 GC3, University of Zurich
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Front-end script for submitting multiple ZODS jobs.
It uses the generic `gc3libs.cmdline.SessionBasedScript` framework.

See the output of ``gzods --help`` for program usage instructions.
"""
__version__ = '2.2.1 version (SVN $Revision$)'
# summary of user-visible changes
__changelog__ = """
  2012-07-17:
    * Initial releases.
  2012-07-31:
    * Added support for submission of multiple files.
  2012-08-06:
    * Added support of restarting the job
"""
__author__ = 'Lukasz Miroslaw <lukasz.miroslaw@uzh.ch>'
__docformat__ = 'reStructuredText'


if __name__ == '__main__':
        import gzods
        gzods.ZodsScript().run()


import os
import xml.dom.minidom
from xml.etree.ElementTree  import ElementTree
import gc3libs
import gc3libs.cmdline

DEFAULT_ZODS_VERSION='0.344'


class GzodsApp(gc3libs.Application):
    """
    This class is derived from gc3libs.Application and defines ZODS app with its input and output files.
    """

    application_name = 'zods'

    def __init__(self, filename, version=DEFAULT_ZODS_VERSION, **extra_args):
        if self.check_input(filename) is None:
                raise gc3libs.exceptions.InputFileError(
                        "Cannot find auxiliary files for input file '%s'."
                        " (Auxiliary files are the average structure '.cif' file,"
                        " the reference intensities file,"
                        " and -optionally- the restart file.)"
                        % (filename,))
        inputFiles = self.check_input(filename)
        gc3libs.log.debug("Detected input files for ZODS: %s, and %s.", filename, [inputFiles])
        myinputs = [filename]
        for inputFile in inputFiles:
                inputFile = os.path.abspath(inputFile)
                myinputs.append(inputFile)

        gc3libs.Application.__init__(
            self,
            tags=[ ("APPS/CHEM/ZODS-%s" % version) ],
            arguments = [
                'mpiexec',
                # these are arguments to `mpiexec`
                "-n", extra_args['requested_cores'],
                # this is the real ZODS command-line
                '$ZODS_BINDIR/simulator', os.path.basename(filename),
            ],
            inputs = myinputs,   # mandatory, inputs are files that will be copied to the site
            outputs = gc3libs.ANY_OUTPUT,           # mandatory
            stdout = "stdout+err.txt",
            join=True,
            **extra_args)


    def terminated(self):
        filename = os.path.join(self.output_dir, self.stdout)
        gc3libs.log.debug("ZODS single job based on %s has TERMINATED", filename)
        for output in self.outputs:
                gc3libs.log.debug("Retrieved the following file from ZODS job %s", output)


#	Detect the following references to external files in input.xml
#	input1:
#	<average_structure>
#          <file format="cif" name="californium_simple_3.cif"/>
#        </average_structure>
#        input2:
#	<reference_intensities file_format="xml" file_name="data.xml"/>
#	input3:
#	<optimization method>
#		<restart file="diff_ev2.xml" />
#	</optimization method>

    def check_input(self,filename):

        if not os.path.exists(filename):
            raise RuntimeError("Input file '%s' DOES NOT exists." % filename)
        basedir = os.path.dirname(filename)
        DOMTree = xml.dom.minidom.parse(filename)
        tree = ElementTree()
        XMLTree = tree.parse(filename)
        cNodes = DOMTree.childNodes

        # Detect <reference_intensities> and <average_structure>
        if len(cNodes[0].getElementsByTagName('reference_intensities')) > 0 and len(cNodes[0].getElementsByTagName('average_structure')) > 0:
            data_file = cNodes[0].getElementsByTagName('reference_intensities')[0].getAttribute('file_name')
            avg_file = cNodes[0].getElementsByTagName('average_structure')[0].getElementsByTagName('file')[0].getAttribute('name')
            data_file = os.path.join(basedir,data_file)
            avg_file = os.path.join(basedir,avg_file)
            gc3libs.log.debug("gzods.py: %s references to the following files: %s and  %s.", filename, avg_file, data_file)
            if os.path.exists(avg_file)  == False or os.path.exists(data_file) == False:
                gc3libs.log.warning("gzods.py: Averaged structure file %s or reference intensities file %s DO NOT exist.", avg_file, data_file)
                raise RuntimeError("gzods.py: Input file '%s' DOES NOT exists." % filename)
            else:
                gc3libs.log.info(
                    "Input file '%s' references"
                    " averaged structure file '%s'"
                    " and reference intesities file '%s'.",
                    filename, avg_file, data_file)
        # Detect optional argument <restart>
                if len(cNodes[0].getElementsByTagName('run_type')) > 0:
                        gc3libs.log.debug("gzods.py: <run_type> tag detected.")
                        if len(cNodes[0].getElementsByTagName('optimization_method')) > 0:
                                gc3libs.log.debug("gzods.py: <optimization_method> tag detected.")
                                if len(cNodes[0].getElementsByTagName('restart')) > 0:
                                        gc3libs.log.debug("gzods.py: <restart> tag and optional restart file detected.")
                                        restart_file = cNodes[0].getElementsByTagName('optimization_method')[0].getElementsByTagName('restart')[0].getAttribute('file')
                                        restart_file = os.path.join(basedir,restart_file)
                                        #restart_file = os.path.abspath(restart_file)
                                        if os.path.exists(restart_file)  == False:
                                                raise gc3libs.exceptions.InputFileError("gzods.py: Input file '%s' references a file '%s' that DOES NOT exists." % (filename, restart_file))
                                        else:
                                                gc3libs.log.info("gzods.py: %s references an optional restart file: %s.", filename, restart_file)

                                        return (data_file, avg_file, restart_file)
                                else:
                                        return (data_file, avg_file)

                        else:
                                return (data_file, avg_file)

class ZodsScript(gc3libs.cmdline.SessionBasedScript):
        """
This application runs the ZODS program on distributed resources.

Any (non-option) argument given on the command line is interpreted as
a directory name; the script scans any directories recursively for
'input*.xml' files, and submits a ZODS job for each input file found.

Job progress is monitored and, when a job is done, its output files
are retrieved back into the same directory where the input '.xml' file is.

The `-c` option allows to set the number of CPU cores for running
parallel jobs; for instance, the following will run parallel ZODS on
file `input.xml` using on 5 CPUs::

  ./gzods input.xml -c 5

        """
        version = "1.0"

        def setup_options(self):
                self.add_param("-R", "--verno", metavar="VERNO",
                               dest="verno", default=DEFAULT_ZODS_VERSION,
                               help="Request the specified version of ZODS"
                               " (default: %(default)s).")

        def new_tasks(self, extra):
           if self.params.args is None or len(self.params.args) == 0:
                self.log.warning(
                        "No arguments given on the command line: no new jobs are created."
                        " Existing jobs will still be managed.")
                return
           tasks=[]
           listOfFiles = self._search_for_input_files(self.params.args, pattern="input*.xml")
           gc3libs.log.debug("List of detected input files for ZODS: %s", listOfFiles)
           for filename in listOfFiles:
               filepath = os.path.abspath(filename)
               extra_args = extra.copy()
               extra_args['version'] = self.params.verno
               # job name comes from input file name, minus the `.xml` ext
               extra_args['jobname'] = os.path.basename(filename)[:-len('.xml')]
               yield GzodsApp(filepath, **extra_args)



#! /usr/bin/env python
#
#   ggeosphere_web.py -- Front-end script for submitting multiple `GeoSphere` jobs.
#
#   Copyright (C) 2013, 2014 GC3, University of Zurich
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Front-end script for submitting multiple `GeoSphere` jobs.
It uses the generic `gc3libs.cmdline.SessionBasedScript` framework.

See the output of ``ggeosphere_web.py --help`` for program usage instructions.
"""

__version__ = '2.2.1 version (SVN $Revision$)'
# summary of user-visible changes
__changelog__ = """

  2014-03-21:
  * Initial release, forked off the ``ggeoshpere`` sources.
"""
__author__ = 'Sergio Maffioletti <sergio.maffioletti@gc3.uzh.ch>'
__docformat__ = 'reStructuredText'


# run script, but allow GC3Pie persistence module to access classes defined here;
# for details, see: http://code.google.com/p/gc3pie/issues/detail?id=95
if __name__ == "__main__":
    import ggeosphere_web
    ggeosphere_web.GeoSphereScript().run()

from pkg_resources import Requirement, resource_filename
import os
import posix

# gc3 library imports
import gc3libs
from gc3libs import Application, Run, Task
from gc3libs.cmdline import SessionBasedScript, executable_file
import gc3libs.utils
from gc3libs.quantity import Memory, kB, MB, GB, Duration, hours, minutes, seconds
from gc3libs.workflow import RetryableTask
import logging

# Default values
DEFAULT_S3CFG="$HOME/.s3cfg"
DEFAULT_CLOUD_ALL="*"
DEFAULT_MAX_RUNNING=10
DEFAULT_MAX_RETRIES=3
DEFAULT_LOG_LEVEL=logging.ERROR
A4MESH_CONFIG_FILE="/etc/a4mesh/a4mesh.cfg"

## custom application class
def validate_config_option(config, option_name, default_value=None):
    if not config.has_option("default",option_name):
        if default_value:
            gc3libs.log.warning("No option '%s' defined. "+
                                "Using default: '%s'" % (option_name, default_value))
            return default_value
        else:
            raise ConfigParser.ParsingError("No option '%s' defined." % option_name)
    else:
        return config.get('default',option_name)

class GeoSphereApplication(Application):
    """
    """

    application_name = 'geosphere'

    def __init__(self, input_dir, working_dir, output_container, **extra_args):
        """
        Prepare remote execution of geosphere wrapper script.
        The resulting Application will associate a remote execution like:

        geosphere_wrapper.sh [options] <input archive> <working dir> <model name>

        Options:
        -g <grok binary file>    path to 'grok' binary. Default in PATH
        -h <hgs binary file>     path to 'hgs' binary. Default in PATH
        -o <S3 url>              store output result on an S3 container
        -d                       enable debug
        """


        self.input_dir = input_dir

        inputs = []
        
        geosphere_wrapper_sh = resource_filename(Requirement.parse("gc3pie"),
                                              "gc3libs/etc/geosphere_wrapper.sh")


        inputs.append((geosphere_wrapper_sh,os.path.basename(geosphere_wrapper_sh)))

        cmd = "./geosphere_wrapper.sh -d "
        

        if extra_args.has_key('s3cfg'):
            inputs.append((extra_args['s3cfg'],
                           "etc/s3cfg"))

        if extra_args.has_key('grok_bin'):
            cmd += "-g %s " % extra_args['grok_bin']

            inputs.append((extra_args['grok_bin'],
                          os.path.join("./bin",
                                       os.path.basename(extra_args['grok_bin']))))

        if extra_args.has_key('hgs_bin'):
            cmd += "-h %s " % extra_args['hgs_bin']

            inputs.append((extra_args['hgs_bin'],
                          os.path.join("./bin",
                                       os.path.basename(extra_args['hgs_bin']))))

        cmd += "%s %s %s" % (input_dir,
                             working_dir,
                             output_container)


        # Application.__init__(
        #     self,
        #     # arguments should mimic the command line interfaca of the command to be
        #     # executed on the remote end
        #     arguments = cmd,
        #     inputs = inputs,
        #     outputs = [],
        #     stdout = 'geosphere.log',
        #     join=True,
        #     **extra_args)


        #XXX: TESTING PURPOSES...
        extra_args['requested_memory'] = 1*MB

        Application.__init__(
            self,
            # the following arguments are mandatory:
            arguments = ["/bin/hostname"],
            inputs = [],
            outputs = [],
            stdout = "stdout.txt",
            join = True,
            **extra_args)

class GeoSphereTask(RetryableTask, gc3libs.utils.Struct):
    """
    Run ``geosphere`` on a given simulation directory until completion.
    """
    def __init__(self, input_dir, working_dir, output_container, **extra_args):
        RetryableTask.__init__(
            self,
            # actual computational job
            GeoSphereApplication(input_dir, working_dir, output_container, **extra_args),
            # keyword arguments
            **extra_args)

## main script class

class GeoSphereScript(SessionBasedScript):
    """
    Loads the configuration file.
    Scans the specified INPUT for model files to simulate with
    ggeosphere. Submit a job for each one found; job progress is
    monitored and, when a job is done, its output files are retrieved back
    into the simulation directory itself.
    
    The ``ggeosphere_web`` is a variant of the ``ggeosphere`` script. It is 
    conceived for a web-based execution. It does not terminate when all input models
    have been processed but it periodically checks whether new models have been added.
    In case, a new simulation is launched.
    It keeps a record of jobs (submitted, executed
    and pending) in a session file (set name with the ``-s`` option); at
    each invocation of the command, the status of all recorded jobs is
    updated, output from finished jobs is collected, and a summary table
    of all known jobs is printed.  

    Options can specify a maximum number of jobs that should be in
    'SUBMITTED' or 'RUNNING' state; ``ggeosphere`` will delay submission of
    newly-created jobs so that this limit is never exceeded.
    """

    def __init__(self):
        SessionBasedScript.__init__(
            self,
            version = __version__, # module version == script version
            application = GeoSphereTask,
            # only display stats for the top-level policy objects
            # (which correspond to the processed files) omit counting
            # actual applications because their number varies over
            # time as checkpointing and re-submission takes place.
            stats_only_for = GeoSphereTask,
            )


    def setup_options(self):
        self.add_param("-c", "--config", 
                       metavar="PATH",
                       action="store",
                       default=A4MESH_CONFIG_FILE,
                       dest="config_file", 
                       help="Location of the configuration file. "+
                       "Default: /etc/a4mesh/a4mesh.cfg")

    def parse_args(self):
        """
        Check presence of input folder (should contains R scripts).
        path to command_file should also be valid.
        """
        
        # Load configuration file
        if not os.path.isfile(self.params.config_file):
            raise gc3libs.exceptions.InvalidUsage("Config file '%s' not found" % self.params.config_file)

        self._read_config_file(self.params.config_file)

        self.input_folder_url = gc3libs.url.Url(os.path.join(self.extra['fs_endpoint'],
                                                             self.extra['model_input']))
        self.output_folder_url = gc3libs.url.Url(os.path.join(self.extra['fs_endpoint'],
                                                              self.extra['model_output']))

        if self.input_folder_url.scheme.lower() == "s3":
            # Then check validity of s3cmd configuration file
            if not os.path.isfile(self.extra['s3cmd_conf_location']):
                raise gc3libs.exceptions.InvalidArgument("S3cmd config file '%s' not found" \
                                                         % self.extra['s3cmd_conf_location'])
        elif self.input_folder_url.scheme.lower() == "file":
            # Check local filesystem
            if not os.path.exists(os.path.abspath(self.input_folder_url.path)):
                raise gc3libs.exceptions.InvalidArgument("Local input folder %s not found" \
                                                         % os.path.abspath(self.input_folder_url.path))

    def _create_task(self, input_dir, **extra_args):
        working_dir = os.path.splitext(os.path.basename(input_dir))[0]
        
        jobname = "a4mesh-%s" % working_dir
    
        extra_args['jobname'] = jobname
        
        # FIXME: ignore SessionBasedScript feature of customizing 
        # output folder
        # extra_args['output_dir'] = self.params.output
        extra_args['output_dir'] = os.path.join(self.extra['simulation_log_location'],jobname)

        extra_args['output_dir'] = extra_args['output_dir'].replace('NAME', jobname)
        extra_args['output_dir'] = extra_args['output_dir'].replace('SESSION', jobname)
        extra_args['output_dir'] = extra_args['output_dir'].replace('DATE', jobname)
        extra_args['output_dir'] = extra_args['output_dir'].replace('TIME', jobname)

        extra_args['s3cfg'] = self.extra['s3cmd_conf_location']
            
        self.log.info("Creating Task for input file: %s" % input_dir)
        
        return GeoSphereTask(
            input_dir,
            working_dir,
            self.output_folder_url,
            **extra_args
        )
        

    def new_tasks(self, extra):

        self.tasks = []

        for input_dir in self._list_S3_container(self.input_folder_url, self.extra['s3cmd_conf_location']):
            self.tasks.append(self._create_task(input_dir, **extra.copy()))

        return self.tasks

    def every_main_loop(self):
        # Check for new tasks
        # eventually add them to self.controller

        gc3libs.log.debug("Reading S3 container for new models")

        for input_dir in self._list_S3_container(self.input_folder_url, self.extra['s3cmd_conf_location']):
            # check if input_dir is already associated to a running Task

            skip = False

            # skip those 'input_dir' that either have been already
            # processed or are already associated to a running task
            for task in self.session:
                if task.input_dir == input_dir:
                    skip = True
                    break

            if not skip:
                # Add as a new task
                gc3libs.log.debug("Found new model to add: '%s'" % input_dir)
                task =  self._create_task(input_dir, **self.extra.copy())
                self.session.add(task)
                self._controller.add(task)

        # # Remove all terminated tasks
        # for task in self.session.tasks:
        #     if task.state in [Run.State.TERMINATED]:
        #         # Remove task from controller
        #         # This should help free memory
        #         self.controlled.remove(task)
        #         self.session.remove(task.id)

    def _main_loop_exitcode(self, stats):
        """
        This works in daemon mode;
        Thus do not end if all simulations have been
        processed. Continue and wait for new models to be uploaded.
        """
        return 4

    def _read_config_file(self, config_file):
        import ConfigParser

        config = ConfigParser.RawConfigParser()
        config.read(self.params.config_file)
        if not config.has_section("default"):
            raise ConfigParser.ParsingError("Section 'default' not found")


        self.extra['s3cmd_conf_location'] = validate_config_option(config, 
                                                                   "s3cmd_conf_location", 
                                                                   DEFAULT_S3CFG)

        self.extra['fs_endpoint'] = validate_config_option(config, 
                                                           "fs_endpoint")
        
        self.extra['model_input'] = validate_config_option(config, 
                                                           "model_input")

        self.extra['model_output'] = validate_config_option(config, 
                                                            "model_output")

        self.extra['computing_resources'] = validate_config_option(config, 
                                                                   "computing_resources", 
                                                                   DEFAULT_CLOUD_ALL)

        self.extra['max_running'] = validate_config_option(config, 
                                                           "max_running", 
                                                           DEFAULT_MAX_RUNNING)

        self.extra['max_retries'] = validate_config_option(config, 
                                                           "max_retries", 
                                                           DEFAULT_MAX_RETRIES)

        self.extra['log_level'] = validate_config_option(config, 
                                                         "log_level", 
                                                         DEFAULT_LOG_LEVEL)

        self.extra['save_simulation_log'] = validate_config_option(config, 
                                                                   "save_simulation_log", 
                                                                   False)

        self.extra['simulation_log_location'] = validate_config_option(config, 
                                                                       "simulation_log_location", 
                                                                       False)


    def _list_S3_container(self, s3_url, s3_config_file=None):
        """
        Use s3cmd command line interface to interact with
        a remote S3-compatible ObjectStore.
        Assumption: 
        . s3cmd configuration file available
        and correctly pointing to the right ObjectStore.
        . s3cmd available in PATH environmental variable.
        . Valid for only 1 S3_URL path
        """

        import subprocess

        # read content of remote S3CMD_URL
        try:
            # 's3cmd ls' should return a list of model archives
            # for each of them bundle archive_name and working_dir
            _command = "s3cmd "
            if s3_config_file:
                _command += " -c %s " % s3_config_file

            _command += " ls %s/"  % str(s3_url)

            _process = subprocess.Popen(_command,
                                        stdout=subprocess.PIPE, 
                                        stderr=subprocess.PIPE,
                                        close_fds=True, shell=True)
                
            (out, err) = _process.communicate()
            exitcode = _process.returncode
            
            if (exitcode != 0):
                gc3libs.log.error("Failed accessing S3 container. "
                                  "Exitcode %s, error %s" % (exitcode,err))
                yield None

            # Parse 'out_list' result and extract all .tgz or .zip archive names

            for s3_obj in out.strip().split("\n"):
                if s3_obj.startswith("DIR"):
                    # it's a S3 directory; ignore
                    continue
                # object string format: '2014-01-09 16:41 3627374 s3://a4mesh/model_1.zip'
                s3_url = s3_obj.split()[3] 
                if(s3_url.startswith("s3://")):
                   yield s3_url

        except Exception, ex:
            gc3libs.log.error("Failed while reading remote S3 container. "+
                              "%s", ex.message)

##################

# Proposal: Associate input models with output results
# For each model stored in personal folder 'input'
# there should be a corresponding result (match by name ?)
# in personal 'output' folder
#
# The web interface builds a representation 'per-model'
# were the user can inspect the model and its associated result
# For those models whose result is not yet available (e.g. in processing)
# an icon is displayed instead.
# End users can remove both the input and the output for a given model;
# semantic of such operations is:
# * remove output: re-simulate the input model
# * remove input: remove also output
# * remove both: remove both


# Configuration files:
# 1. gc3pie.conf
# 2. s3cmd.conf
# 3. ggeosphere_web.conf

# Details of ggeosphere_web.conf
# S3 locations: input / output
# Computing cloud infrastructure: [GC3Pie resource(s)]
# Max number of simultaneous simulations
# Max number of retries per failed simulation
# Verbosity level
# Retain log information for each completed simulation [True/False]
# location where info on sompleted simulation should be stored
"""
example:

[default]
gc3pie_conf_location = /etc/gc3/gc3pie.conf
s3cmd_conf_location = /etc/s3cmd/s3cfg
s3_endpoint = s3://models
s3_input = 'input'
s3_output = 'output'
computing_resources = a4mesh.uzh, a4mesh.zhaw, a4mesh.hes-so
max_running = 10
max_retries = 2
log_level = DEBUG
save_simulation_log = False
simulation_log_location = '/tmp/simulation_logs/

"""







#! /usr/bin/env python
#
"""
Manipulation of quantities with units attached with automated
conversion among compatible units.

For details and the discussion leading up to this,
see: `<http://code.google.com/p/gc3pie/issues/detail?id=47>`

"""
# Copyright (C) 2011, 2012, 2013, GC3, University of Zurich. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
__docformat__ = 'reStructuredText'
__version__ = '$Revision$'


# stdlib imports
import datetime
import operator
import re
import types

# GC3Libs imports
from gc3libs.utils import defproperty


## utility functions

_QTY_RE = re.compile(
    r'(?P<amount>[+-]?([0-9]+(\.[0-9]+)?|\.[0-9]+)(E[+-]?[0-9]+)?)'
    r'\s*'
    r'(?P<unit>[a-z]+)?',
    re.I | re.X)

def _split_amount_and_unit(val, default_unit=None, allow=None):
    """
    Split `val` into amount and measurement unit.

    The string to be parsed should consist of a number, followed by a
    unit specification. The number and the unit may be separated by 0
    or more spaces.

      >>> _split_amount_and_unit('7 s')
      (7.0, 's')
      >>> _split_amount_and_unit('7s')
      (7.0, 's')
      >>> _split_amount_and_unit('-7.0s')
      (-7.0, 's')

    If `val` does not conform to this syntax, a `ValueError` will
    be raised::

      >>> _split_amount_and_unit('foo')
      Traceback (most recent call last):
        ...
      ValueError: Cannot parse quantity 'foo'

    If string `val` only specifies an amount (e.g., ``42``) with no
    unit, then the optional argument `default_unit` provides one::

      >>> _split_amount_and_unit('7', 'min')
      (7.0, 'min')

    Note the amount is a floating point number, whereas the unit is
    always a string.  By default, no validity checks are performed
    on the 'unit' part, which can be any word::

      >>> _split_amount_and_unit('7sins')
      (7.0, 'sins')

    The optional argument `allow` restricts the resulting quantity to
    be a one of the specified units; if it's not, a `ValueError`
    exception is raised.

      >>> _split_amount_and_unit('42 GB', allow=['s', 'min', 'hour'])
      Traceback (most recent call last):
        ...
      ValueError: Unit 'GB' is not allowed here: only 'hour','min','s' are.

    """
    match = _QTY_RE.search(val)
    if not match:
        raise ValueError("Cannot parse quantity '%s'" % val)
    amount = float(match.group('amount'))
    unit = match.group('unit')
    if unit is None:
        unit = default_unit
    if allow is not None:
        if unit not in allow:
            raise ValueError("Unit '%s' is not allowed here: only %s are."
                             % (unit, str.join(",", [("'%s'" % a) for a in sorted(allow)])))
    return (amount, unit)


# since the code for `_Quantity` comparison methods is basically the
# same for all methods, we use a decorator-based approach to reduce
# boilerplate code...  Oh, how do I long for LISP macros! :-)
def _make_comparison_function(op, domain):
    """
    Return a function that compares ensures that the two operands
    belong to the same class, and then compares their value (obtained
    by converting to `domain`) with the passed relational operator `op`.

    Completely discards the function being decorated.
    """
    def decorate(fn):
        def cmp_fn(self, other):
            assert self.__class__ is other.__class__, (
                "Cannot compare '%s' with '%s':"
                " Can only compare homogeneous quantities!"
                % (self.__class__.__name__, other.__class__.__name__))
            return op(self.amount(unit=self.base, conv=domain), other.amount(unit=self.base, conv=domain))
        return cmp_fn
    return decorate


class _Quantity(object):
    """
    Represent a dimensioned value.

    A dimensioned value is a pair formed by a magnitude and a
    measurement unit.

    |  >>> qty1 = Memory('1kB')
    |  >>> qty2 = Memory('2 kB')

    This class also implements the basic arithmetic on quantities,
    i.e., multiplication by a scalar value and taking the ratio of two
    homogeneous quantities::

    |  >>> 2 * qty1
    |  2 kB
    |  >>> 2 * qty1 == qty2
    |  True
    |  >>> qty2 / qty1
    |  2

    .. note::

      Representing dimensioned quantities is a multi-faceted problem;
      we make two simplifying assumptions in representing quantities:

      1. *Every quantity is representable as an integral multiple
         of a base quantity* (the base unit).  This holds true for
         representation of RAM memory amounts and time lapses on
         64-bit systems, which is enough for our purposes.

      2. *Homogeneous quantities are instances of the same class.*

      For more background on the 'Quantity' pattern and the issues
      in implementing it, see:

      - http://martinfowler.com/eaaDev/quantity.html
      - http://mail.python.org/pipermail/python-ideas/2010-March/006894.html
      - http://mgrzyb.blogspot.ch/2011/08/quantity-pattern-lessons-learned-from.html

    """

    __slots__ = (
        '_amount',
        '_base',
        '_name',
        '_unit',
        '_UNITS',
        )

    def amount(self, unit=None, conv=(lambda value: value)):
        """
        Return the (numerical) amount of this quantity.

        If the optional argument `unit` is specified, return the
        amount in the given units; otherwise, the amount is expressed
        in the units stored in the `unit`:attr: attribute.

        The optional argument `conv` can be used to convert the
        amounts to a specific numerical domain.  For example,
        `conv=int` returns the amount as an integer multiple of the
        unit amount; in particular, the returned amount will be `0` if
        the quantity is less than one unit amount.
        """
        if unit is None:
            unit = self.unit
        return (conv(self._amount) / conv(unit._amount))

    @defproperty
    def base():
        """
        Return the base unit for all quantities in this class.
        The amount of any quantity is internally stored as a multiple
        of this base unit.

        This is a read-only class-level attribute.
        """
        def fget(self):
            return self._base
        return locals()

    @defproperty
    def unit():
        """
        The unit the amount of this quantity is expressed in.

        This is a read-only attribute: once set in the constructor, it
        cannot be changed.
        """
        def fget(self):
            return self._unit
        return locals()

    @defproperty
    def name():
        """
        The name of this quantity, or `None`.

        Units are named quantities; for any other quantity, the
        attribute `name` evaluates to `None`.

        This is a read-only attribute: once set in the constructor, it
        cannot be changed.
        """
        def fget(self):
            return self._name
        return locals()

    ## instance construction

    _UNITS = { }
    """
    A registry of valid units (by name).

    Units that are listed here are recognized as valid during
    construction of a quantity from a string (see method
    `_new_from_string`:meth:).
    """

    @classmethod
    def register(cls, unit):
        """
        Register a new named quantity, i.e., a unit.
        """
        cls._UNITS[unit.name] = unit


    # since we are subclassing a builtin type,
    # we need to provide `__new__`, not `__init__`
    def __new__(cls, val, unit=None, name=None):
        # dispatch to actual constructor depending on the type of `val`
        if isinstance(val, types.StringTypes):
            new = cls._new_from_string(val)
        elif isinstance(val, _Quantity):
            new = cls._new_from_amount_and_unit(val.amount(), val.unit)
        elif unit is cls:
            # special case to bootstrap the base unit
            new = super(_Quantity, cls).__new__(cls)
            new._amount = 1
            new._unit = new
        else:
            # default: quantity is int(val)*unit
            assert unit is not None, ("Cannot construct a quantity from amount alone.")
            assert unit.name in cls._UNITS, (
                "Unit '%s' not allowed in '%s' quantity: only %s are."
                % (unit.name, cls.__name__,
                   str.join(',', [("'%s'" % u) for u in cls._UNITS])))
            new = cls._new_from_amount_and_unit(val, unit)
        new._name = name
        if name is not None:
            cls.register(new)
        return new

    @classmethod
    def _new_from_amount_and_unit(cls, amount, unit):
        new = super(_Quantity, cls).__new__(cls)
        new._amount = amount*unit.amount(cls._base)
        new._unit = unit
        new._name = None
        return new

    @classmethod
    def _new_from_string(cls, val):
        amount, unitname = _split_amount_and_unit(val, allow=cls._UNITS)
        unit = cls._UNITS[unitname]
        return cls._new_from_amount_and_unit(amount, unit)

    def __getnewargs__(self):
        # use the string serialization, as the `(amount, unit, name)`
        # constructor could lead to infinite recursion ("unit" could
        # be the very class itself that we are going to serialize)
        return (("%d %s" % (self.amount(), self.unit.name)), self.name)


    ## string representation
    def to_str(self, fmt, unit=None, conv=(lambda value: value)):
        """
        Return a string representation of the quantity.

        Arguments `fmt` and `unit` influence how the string
        representation is formed::

        |  >>> qty = Memory('1024kB')
        |  >>> qty.format('%d [%s]')
        |  '1024 [kB]'
        |  >>> qty.format('%d [%s]', unit=MB)
        |  '1 [MB]'
        |  >>> qty.format('%g%s', unit=GB)
        |  0.001GB

        :param str fmt:
          Format string, with ``%``-style specifiers.

        :param unit:
          Unit quantity; the numeric amount is a multiple of this unit.

        :param conv:
          Passed unchanged to the `amount`:meth: method (which see).
        """
        if unit is None:
            unit = self.unit
        try:
            return (fmt % (self.amount(unit, conv=conv), unit.name))
        except TypeError: # not all arguments converted
            return (fmt % self.amount(unit, conv=conv)) + unit.name

    def __str__(self):
        """
        Return a human-readable string representation of this quantity.
        """
        return self.to_str("%g%s")

    def __repr__(self):
        """Return a string representation that can be read back with `eval()`."""
        if hasattr(self.unit, 'name'):
            return ('%s(%g, unit=%s)'
                    % (self.__class__.__name__, self.amount(), self.unit.name))
        else:
            return ('%s(%g, unit=<%s object at 0x%x>)'
                    % (self.__class__.__name__, self.amount(), self.unit.__class__.__name__, id(self.unit)))

    ## arithmetic: allow multiplication by a scalar, and division by a quantity (of the same kind)
    @staticmethod
    def _smallest_unit(self, other):
        """
        Return the smallest between `self.unit` and `other.unit`.
        """
        self_unit_amount = self.unit.amount(self.base)
        other_unit_amount = other.unit.amount(self.base)
        if self_unit_amount <= other_unit_amount:
            return self.unit
        else:
            return other.unit

    @classmethod
    def _largest_nonfractional_unit(cls, amount):
        """
        Return largest unit such that `amount` is larger than 1 when
        expressed in that unit.
        """
        units = sorted(cls._UNITS.values())
        prev = units[0].base
        for unit in units:
            if prev.amount(unit.base) <= amount < unit.amount(unit.base):
                return prev
            prev = unit
        return unit

    def __add__(self, other):
        assert isinstance(other, self.__class__), \
               ("Cannot add '%s' to '%s':"
                " can sum only homogeneous quantities."
                % (self.__class__.__name__, other.__class__.__name__))
        unit = self._smallest_unit(self, other)
        return self._new_from_amount_and_unit(
            (self.amount(self.base) + other.amount(self.base)) / unit.amount(self.base),
            unit=unit)

    def __sub__(self, other):
        assert isinstance(other, self.__class__), \
               ("Cannot subtract '%s' from '%s':"
                " can only operate on homogeneous quantities."
                % (self.__class__.__name__, other.__class__.__name__))
        unit = self._smallest_unit(self, other)
        return self._new_from_amount_and_unit(
            (self.amount(self.base) - other.amount(self.base)) / unit.amount(self.base),
            unit=unit)

    def __mul__(self, coeff):
        if __debug__:
            try:
                float(coeff)
            except (TypeError, ValueError):
                raise TypeError("Cannot multiply '%s' and '%s':"
                                " can only multiply '%s' by a pure number."
                                ""% (coeff.__class__.__name__,
                                     self.__class__.__name__,
                                     self.__class__.__name__))
        return self._new_from_amount_and_unit(coeff * self.amount(), self.unit)
    __rmul__ = __mul__

    def __div__(self, other):
        """
        Return the ratio of two quantities (as a floating-point number),
        or divide a quantity by the specified amount.
        """
        try:
            # the quotient of two (homogeneous) quantities is a ratio (pure number)
            return (self.amount(self.base, conv=float) / other.amount(self.base, conv=float))
        except AttributeError:
            # we could really return `self * (1.0/other)`, but we want
            # to set the unit to a possibly smaller one (see
            # `_get_best_unit` above) to have a better "human" representation
            try:
                amount = self.amount(self.base, conv=float) / float(other)
            except TypeError:
                raise TypeError(
                    "Cannot multiply '%s' and '%s': can only take"
                    " the ratio of '%s' and a pure number or an"
                    " homogeneous quantity." % (coeff.__class__.__name__,
                                                self.__class__.__name__,
                                                self.__class__.__name__))
            unit = self._largest_nonfractional_unit(amount)
            return self._new_from_amount_and_unit(amount / unit.amount(self.base), unit)

    def __floordiv__(self, other):
        """Return the ratio of two quantities (as a whole number)."""
        assert isinstance(other, self.__class__), \
               ("Cannot divide '%s' by '%s':"
                " can only take the ratio of homogeneous quantities."
                % (self.__class__.__name__, other.__class__.__name__))
        # the quotient of two (homogeneous) quantities is a ratio (pure number)
        return (self.amount(self.base, conv=int) / other.amount(self.base, conv=int))

    def __radd__(self, other):
        """
        Allow summing with a null value, even if non-quantity.
        Summing with any other value raises a `TypeError`.

        This is provided only to support built-in reducers like `sum`,
        which start with a zero value and then sum all the arguments
        to it.
        """
        if other == 0 or other == 0.0:
            return self
        else:
            raise TypeError(
                "Unsupported operands for +: %s (type '%s') and %s (type '%s')"
                % (other, type(other), self, type(self)))

    ## rich comparison operators, to ensure only homogeneous quantities are compared
    @_make_comparison_function(operator.gt, int)
    def __gt__(self, other):
        pass

    @_make_comparison_function(operator.ge, int)
    def __ge__(self, other):
        pass

    @_make_comparison_function(operator.eq, int)
    def __eq__(self, other):
        pass

    @_make_comparison_function(operator.ne, int)
    def __ne__(self, other):
        pass

    @_make_comparison_function(operator.le, int)
    def __le__(self, other):
        pass

    @_make_comparison_function(operator.lt, int)
    def __lt__(self, other):
        pass


class Quantity(object):
    """
    Metaclass for creating quantity classes.

    This factory creates subclasses of `_Quantity`:class: and
    bootstraps the base unit.

    The name of the base unit is given as argument to the metaclass
    instance::

      >>> class Memory1(object):
      ...   __metaclass__ = Quantity('B')
      ...
      >>> B = Memory1('1 B')
      >>> print (2*B)
      2B

    Optional keyword arguments create additional units; the argument
    key gives the unit name, and its value gives the ratio of the new
    unit to the base unit.  For example::

      >>> class Memory2(object):
      ...   __metaclass__ = Quantity('B', kB=1000, MB=1000*1000)
      ...
      >>> a_thousand_kB = Memory2('1000kB')
      >>> MB = Memory2('1   MB')
      >>> a_thousand_kB == MB
      True

    Note that the units (base and additional) are also available as
    class attributes for easier referencing in Python code::

      >>> a_thousand_kB == Memory2.MB
      True

    """
    def __init__(self, base_unit_name, **other_units):
        self.base_unit_name = base_unit_name
        self.other_units = other_units

    def __call__(self, clsname, bases, attrs):
        bases = tuple([_Quantity] + list(bases))
        newcls = type(clsname, bases, attrs)
        newcls._UNITS = dict()
        # create base unit
        base = newcls(1, unit=newcls, name=self.base_unit_name)
        newcls._base = base
        # alias the human-readable name to it
        newcls.register(base)
        setattr(newcls, self.base_unit_name, base)
        # create additional units and add them as class attributes
        for name, amount in self.other_units.iteritems():
            unit = newcls(amount, unit=newcls._base, name=name)
            # make new units default to self when printing the amount
            unit._unit = unit
            newcls.register(unit)
            setattr(newcls, name, unit)
        return newcls


class Memory(object):
    """
    Represent an amount of RAM.

    Construction of a memory quantity can be done by parsing a string
    specification (amount followed by unit)::

        >>> byte = Memory('1 B')
        >>> kilobyte = Memory('1 kB')

    A new quantity can also be defined as a multiple of an existing
    one::

        >>> a_thousand_kB = 1000*kilobyte

    The base-10 units (up to TB, Terabytes) and base-2 (up to TiB,
    TiBiBytes) are available as attributes of the `Memory` class.
    This allows for a third way of constructing quantity objects,
    i.e., by passing the amount and the unit separately to the
    constructor::

        >>> a_megabyte = Memory(1, Memory.MB)
        >>> a_mibibyte = Memory(1, Memory.MiB)

        >>> a_gigabyte = 1*Memory.GB
        >>> a_gibibyte = 1*Memory.GiB

        >>> two_terabytes = 2*Memory.TB
        >>> two_tibibytes = 2*Memory.TiB

    Two memory quantities are equal if they indicate the exact same
    amount in bytes::

        >>> kilobyte == 1000*byte
        True
        >>> a_megabyte == a_mibibyte
        False
        >>> a_megabyte < a_mibibyte
        True
        >>> a_megabyte > a_gigabyte
        False

    Basic arithmetic is possible with memory quantities::

        >>> two_bytes = byte + byte
        >>> two_bytes == 2*byte
        True
        >>> half_gigabyte = a_gigabyte / 2
        >>> half_gigabyte
        Memory(476.837, unit=MiB)

    The ratio of two memory quantities is correctly computed as a pure
    (floating-point) number::

        >>> a_gigabyte / a_megabyte
        1000.0

    It is also possible to add memory quantities defined with
    different units; the result is naturally expressed in the smaller
    unit of the two::

        >>> one_gigabyte_and_half = 1*Memory.GB + 500*Memory.MB
        >>> one_gigabyte_and_half
        Memory(1500, unit=MB)

    Note that the two unit class and numeric amount are accessible through
    the `unit` and `amount`:meth: attributes::

        >>> one_gigabyte_and_half.unit
        Memory(1, unit=MB)
        >>> one_gigabyte_and_half.amount()
        1500

    The `amount`:meth: method accepts an optional specification of an
    alternate unit to express the amount into::

        >>> one_gigabyte_and_half.amount(Memory.GB)
        1

    An optional `conv` argument is available to specify a numerical
    domain for conversion, in case the default integer arithmetic
    is not precise enough::

        >>> one_gigabyte_and_half.amount(Memory.GB, conv=float)
        1.5

    The `to_str`:meth: method allows representing a quantity as a
    string, and provides choice of the output format and unit.  The
    format string should contain exactly two ``%``-specifiers: the
    first one is used to format the numerical amount, and the second
    one to format the measurement unit name.

    By default, the unit used originally for defining the quantity is
    used::

        >>> a_megabyte.to_str('%d [%s]')
        '1 [MB]'

    This can be overridden by specifying an optional second argument
    `unit`::

        >>> a_megabyte.to_str('%d [%s]', unit=Memory.kB)
        '1000 [kB]'

    A third optional argument `conv` can set the numerical type to be
    used for conversion computations::

        >>> a_megabyte.to_str('%g%s', unit=Memory.GB, conv=float)
        '0.001GB'

    The default numerical type is `int`, which in particular implies
    that you get a null amount if the requested unit is larger than
    the quantity::

        >>> a_megabyte.to_str('%g%s', unit=Memory.GB, conv=int)
        '0GB'

    Conversion to string uses the unit originally used for defining
    the quantity and the ``%g%s`` format::

        >>> str(a_megabyte)
        '1MB'

    """
    __metaclass__ = Quantity(
        # base unit is "bytes"; use the symbol 'B', although this is not the SI usage.
        'B',
        # 10-base units
        kB=1000,
        MB=1000*1000,
        GB=1000*1000*1000,
        TB=1000*1000*1000*1000,
        PB=1000*1000*1000*1000*1000,
        # binary base units
        KiB=1024,                # KiBiByte
        MiB=1024*1024,           # MiBiByte
        GiB=1024*1024*1024,      # GiBiByte
        TiB=1024*1024*1024*1024, # TiBiByte
        PiB=1024*1024*1024*1024*1024, # PiBiByte
        )


class Duration(object):
    """
    Represent the duration of a time lapse.

    Construction of a duration can be done by parsing a string
    specification; several formats are accepted:

    * A duration is an aggregate of days, hours, minutes and seconds::

        >>> l3 = Duration('1day 4hours 9minutes 16seconds')
        >>> l3.amount(Duration.s) # convert to seconds
        101356

    * Any of the terms can be omitted (in which case it defaults
      to zero)::

        >>> l4 = Duration('1day 4hours 16seconds')
        >>> l4 == l3 - Duration('9 minutes')
        True

    * The unit names can be singular or plural, and any amount of
      space can be added between the time unit name and the
      associated amount::

        >>> l5 = Duration('3 hour 42 minute')
        >>> l6 = Duration('3 hours 42 minutes')
        >>> l7 = Duration('3hours 42minutes')
        >>> l5 == l6 == l7
        True

    * Unit names can also be abbreviated using just the leading
      letter::

        >>> l8 = Duration('3h 42m')
        >>> l9 = Duration('3h42m')
        >>> l8 == l9
        True

    * The abbreviated formats HH:MM:SS and DD:HH:MM:SS are also
      accepted::

        >>> # 1 hour + 1 minute + 1 second
        >>> l1 = Duration('01:01:01')
        >>> l1 == Duration('3661 s')
        True

        >>> # 1 day, 2 hours, 3 minutes, 4 seconds
        >>> l2 = Duration('01:02:03:04')
        >>> l2.amount(Duration.s)
        93784

      However, the formats HH:MM and MM:SS are rejected as ambiguous::

        >>> # is this hours:minutes or minutes:seconds ?
        >>> l0 = Duration('01:02')
        Traceback (most recent call last):
          ...
        ValueError: Duration '01:02' is ambiguous: use '1m 2s' for 1 minutes and 2 seconds, or '1h 2m' for 1 hours and 2 minutes.

    * Finally, you can specify a duration like any other quantity,
      as an integral amount of a given time unit::

        >>> l1 = Duration('1 day')
        >>> l2 = Duration('86400 s')
        >>> l1 == l2
        True

    A new quantity can also be defined as a multiple of an existing
    one::

        >>> an_hour = Duration('1 hour')
        >>> a_day = 24 * an_hour
        >>> a_day.amount(Duration.h)
        24

    The quantities ``Duration.hours``, ``Duration.minutes`` and
    ``Duration.seconds`` (and their single-letter abbreviations ``h``,
    ``m``, ``s``) are pre-defined with their obvious meaning.

    Also module-level aliases ``hours``, ``minutes`` and ``seconds``
    (and the one-letter forms) are available::

      >>> a_day1 = 24*hours
      >>> a_day2 = 1440*minutes
      >>> a_day3 = 86400*seconds

    This allows for yet another way of constructing duration objects,
    i.e., by passing the amount and the unit separately to the
    constructor::

      >>> a_day4 = Duration(24, hours)

    Two durations are equal if they indicate the exact same
    amount in seconds::

      >>> a_day1 == a_day2
      True
      >>> a_day1.amount(s)
      86400
      >>> a_day2.amount(s)
      86400

      >>> a_day == an_hour
      False
      >>> a_day.amount(minutes)
      1440
      >>> an_hour.amount(minutes)
      60

    Basic arithmetic is possible with durations::

      >>> two_hours = an_hour + an_hour
      >>> two_hours == 2*an_hour
      True

      >>> one_hour = two_hours - an_hour
      >>> one_hour.amount(seconds)
      3600

    It is also possible to add duration quantities defined with
    different units; the result is naturally expressed in the smaller
    unit of the two::

        >>> one_hour_and_half = an_hour + 30*minutes
        >>> one_hour_and_half
        Duration(90, unit=m)

    Note that the two unit class and numeric amount are accessible through
    the `unit` and `amount`:meth: attributes::

        >>> one_hour_and_half.unit
        Duration(1, unit=m)
        >>> one_hour_and_half.amount()
        90

    The `amount`:meth: method accepts an optional specification of an
    alternate unit to express the amount into::

        >>> one_hour_and_half.amount(Duration.hours)
        1

    An optional `conv` argument is available to specify a numerical
    domain for conversion, in case the default integer arithmetic
    is not precise enough::

        >>> one_hour_and_half.amount(Duration.hours, conv=float)
        1.5

    The `to_str`:meth: method allows representing a duration as a
    string, and provides choice of the output format and unit.  The
    format string should contain exactly two ``%``-specifiers: the
    first one is used to format the numerical amount, and the second
    one to format the measurement unit name.

    By default, the unit used originally for defining the quantity is
    used::

        >>> an_hour.to_str('%d [%s]')
        '1 [hour]'

    This can be overridden by specifying an optional second argument
    `unit`::

        >>> an_hour.to_str('%d [%s]', unit=Duration.m)
        '60 [m]'

    A third optional argument `conv` can set the numerical type to be
    used for conversion computations::

        >>> an_hour.to_str('%.1f [%s]', unit=Duration.m, conv=float)
        '60.0 [m]'

    The default numerical type is `int`, which in particular implies
    that you get a null amount if the requested unit is larger than
    the quantity::

        >>> an_hour.to_str('%d [%s]', unit=Duration.days)
        '0 [days]'

    Conversion to string uses the unit originally used for defining
    the quantity and the ``%g%s`` format::

        >>> str(an_hour)
        '1hour'

    """
    __metaclass__ = Quantity(
        # base unit is nanoseconds; use the SI symbol 'ns'
        'ns',
        # alternate spellings
        nanosec=1,
        nanoseconds=1,
        # microsecond(s)
        ms=1000,
        microsec=1000,
        microseconds=1000,
        # seconds(s)
        s=10**6,
        sec=10**6,
        secs=10**6,
        second=10**6,
        seconds=10**6,
        # minute(s)
        m=60 * 10**6,
        min=60 * 10**6,
        mins=60 * 10**6,
        minute=60 * 10**6,
        minutes=60 * 10**6,
        # hour(s)
        h=60*60 * 10**6,
        hr=60*60 * 10**6,
        hrs=60*60 * 10**6,
        hour=60*60 * 10**6,
        hours=60*60 * 10**6,
        # day(s)
        d=24*60*60 * 10**6,
        day=24*60*60 * 10**6,
        days=24*60*60 * 10**6,
        )

    # override ctor to hook `_new_from_timedelta` in
    def __new__(cls, val, unit=None, name=None):
        if isinstance(val, datetime.timedelta):
            return cls._new_from_timedelta(val)
        else:
            return _Quantity.__new__(cls, val, unit, name)

    # override `_new_from_string` to allow more elaborate parsing of time specs
    @classmethod
    def _new_from_string(cls, val):
        """
        Parse string `val` and return a corresponding `Duration` object.

        See the `Duration`:class: documentation for a list of accepted
        spec formats.
        """
        match = _TIMESPEC_RE.match(val)
        if match and match.end() > match.start():
            # build duration in seconds by summing contributions;
            # use last unit as overall unit
            lapse = 0
            last_unit = Duration.s
            for name, unit_lapse, unit in _TIMESPEC_VALS:
                if match.group(name):
                    lapse += int(match.group(name)) * unit_lapse
                    last_unit = unit
            return cls._new_from_amount_and_unit(
                amount=(lapse / last_unit.amount(Duration.s)),
                unit=last_unit)
        elif ':' in val:
            # since `val` didn't match `_TIMESPEC_RE`, then it must
            # have the form HH:MM or MM:SS, which is ambiguous
            try:
                val1, val2 = val.split(':')
                val1 = int(val1)
                val2 = int(val2)
            except ValueError:
                # `val1` or `val2` cannot be converted to int
                raise ValueError("Cannot parse %r as a time duration." % val)
            raise ValueError("Duration '%s' is ambiguous:"
                             " use '%dm %ds' for %d minutes and %d seconds,"
                             " or '%dh %dm' for %d hours and %d minutes."
                             % (val,
                                val1, val2, val1, val2,
                                val1, val2, val1, val2))
        else:
            # strings like `30 seconds` can be parsed by `_Quantity._new_from_string`;
            # see http://blogs.gnome.org/jamesh/2005/06/23/overriding-class-methods-in-python/
            return super(Duration, cls)._new_from_string(val)

    @classmethod
    def _new_from_timedelta(cls, td):
        """
        Return a duration expressing the same time amount as the Python
        `datetime.timedelta` object `td`.

        Examples::

            >>> from datetime import timedelta
            >>> td1 = timedelta(days=1)
            >>> l1 = Duration(td1)
            >>> l1.amount(Duration.s)
            86400
            >>> l1 == Duration('1 day')
            True

            >>> td2 = timedelta(hours=1, minutes=1, seconds=1)
            >>> l2 = Duration(td2)
            >>> l2.amount(Duration.s)
            3661
        """
        try:
            # Python 2.7 onwards
            return cls._new_from_amount_and_unit(int(td.total_seconds()), Duration.s)
        except AttributeError:
            return cls._new_from_amount_and_unit(
                int(td.seconds + td.days * 24 * 3600),
                Duration.s)

    def to_timedelta(duration):
        """
        Convert a duration into a Python `datetime.timedelta` object.

        This is useful to operate on Python's `datetime.time` and
        `datetime.date` objects, which can be added or subtracted to
        `datetime.timedelta`.
        """
        return datetime.timedelta(seconds=duration.amount(Duration.s))

# needed by `Duration._new_from_string`
_TIMESPEC_RE = re.compile(
    r'('
    # 1. allow HH:MM:SS or DD:HH:MM:SS; the form XX:YY is rejected because of ambiguity
    r'((?P<days1>[0-9]+):)?(?P<hours1>[0-9]+):(?P<minutes1>[0-9]+):(?P<secs1>[0-9]+)'
    r'|'
    # 2. allow `4days 3hrs`, `1 hour 4 minutes` and abbreviations `1d 2h 4m`
    r'((?P<days2>[0-9]+) \s* d(ays?))? \s*'
    r'((?P<hours2>[0-9]+) \s* h((ou)?rs?))? \s*'
    r'((?P<minutes2>[0-9]+) \s* m(in(ute)?s?))? \s*'
    r'((?P<secs2>[0-9]+) \s* s(ecs?))?'
    # 3. everything else is forwarded to `_Quantity._new_from_string`
    r')',
    re.X|re.I)
_TIMESPEC_VALS = [
    # regexp group name
    # |          unit time lapse as seconds
    # |          |         corresponding `Duration` unit
    # |          |         |
    ('days1',    24*60*60, Duration.day),
    ('hours1',   60*60,    Duration.hour),
    ('minutes1', 60,       Duration.minute),
    ('secs1',    1,        Duration.s),
    ('days2',    24*60*60, Duration.day),
    ('hours2',   60*60,    Duration.hour),
    ('minutes2', 60,       Duration.minute),
    ('secs2',    1,        Duration.s),
    ]


## aliases for common units

B = Memory.B
byte = B
bytes = B

kB = Memory.kB
MB = Memory.MB
GB = Memory.GB
TB = Memory.TB

KiB = Memory.KiB
MiB = Memory.MiB
GiB = Memory.GiB
TiB = Memory.TiB

s = Duration.s
secs = s
seconds = s

m = Duration.m
mins = m
minutes = m

h = Duration.h
hrs = h
hours = h

d = Duration.d
days = d


## main: run tests

if "__main__" == __name__:
    import doctest
    doctest.testmod(name="quantity",
                    optionflags=doctest.NORMALIZE_WHITESPACE)



#! /usr/bin/env python
"""
Generic Python programming utility functions.

This module collects general utility functions, not specifically
related to GC3Libs.  A good rule of thumb for determining if a
function or class belongs in here is the following: place a function
or class in this module if you could copy its code into the
sources of a different project and it would not stop working.
"""
# Copyright (C) 2009-2014 GC3, University of Zurich. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
__docformat__ = 'reStructuredText'
__version__ = '2.2.1 version (SVN $Revision: 3977 $)'


import contextlib
import itertools
import os
import os.path
import posix
import random
import re
import shutil
import sys
import tempfile
import time
import cStringIO as StringIO
import UserDict


from gc3libs.compat._collections import defaultdict, OrderedDict
import gc3libs.compat.functools as functools
import gc3libs.compat.lockfile as lockfile

import gc3libs
import gc3libs.exceptions
import gc3libs.debug

# This fixes an issue with Python 2.4, which does not have
# `shutl.WindowsError`
try:
    WindowsError = shutil.WindowsError
except AttributeError:
    class NeverUsedException(Exception):
        """this exception should never be raised"""
    WindowsError = NeverUsedException

def backup(path):
    """
    Rename the filesystem entry at `path` by appending a unique
    numerical suffix; return new name.

    For example,

    1. create a test file:

      >>> import tempfile
      >>> path = tempfile.mkstemp()[1]

    2. then make a backup of it; the backup will end in ``.~1~``:

      >>> path1 = backup(path)
      >>> os.path.exists(path + '.~1~')
      True

    3. re-create the file, and make a second backup: this time the
    file will be renamed with a ``.~2~`` extension:

      >>> open(path, 'w').close()
      >>> path2 = backup(path)
      >>> os.path.exists(path + '.~2~')
      True

    cleaning up tests

      >>> os.remove(path+'.~1~')
      >>> os.remove(path+'.~2~')

    """
    parent_dir = (os.path.dirname(path) or os.getcwd())
    prefix = os.path.basename(path) + '.~'
    p = len(prefix)
    suffix = 1
    for name in [ entry for entry in os.listdir(parent_dir)
                  if (entry.startswith(prefix) and entry.endswith('~')) ]:
        try:
            n = int(name[p:-1])
            suffix = max(suffix, n+1)
        except ValueError:
            # ignore non-numeric suffixes
            pass
    new_path = "%s.~%d~" % (path, suffix)
    os.rename(path, new_path)
    return new_path

def data_merge(source_path, destination_path):
    # mv --update -v in/* ../in/
    pass


def basename_sans(path):
    """
    Return base name without the extension.
    """
    return os.path.splitext(os.path.basename(path))[0]


def cache_for(lapse):
    """
    Cache the result of a (nullary) method invocation for a given
    amount of time. Use as a decorator on object methods whose results
    are to be cached.

    Store the result of the first invocation of the decorated
    method; if another invocation happens before `lapse` seconds
    have passed, return the cached value instead of calling the real
    function again.  If a new call happens after the grace period has
    expired, call the real function and store the result in the cache.

    **Note:** Do not use with methods that take keyword arguments, as
    they will be discarded! In addition, arguments are compared to
    elements in the cache by *identity*, so that invoking the same
    method with equal but distinct object will result in two separate
    copies of the result being computed and stored in the cache.

    Cache results and timestamps are stored into the objects'
    `_cache_value` and `_cache_last_updated` attributes, so the caches
    are destroyed with the object when it goes out of scope.

    The working of the cached method can be demonstrated by the
    following simple code::

        >>> class X(object):
        ...     def __init__(self):
        ...         self.times = 0
        ...     @cache_for(2)
        ...     def foo(self):
        ...             self.times += 1
        ...             return ("times effectively run: %d" % self.times)
        >>> x = X()
        >>> x.foo()
        'times effectively run: 1'
        >>> x.foo()
        'times effectively run: 1'
        >>> time.sleep(3)
        >>> x.foo()
        'times effectively run: 2'

    """
    def decorator(fn):
        @functools.wraps(fn)
        def wrapper(obj, *args):
            now = time.time()
            key = (fn, tuple(id(arg) for arg in args))
            try:
                update = ((now - obj._cache_last_updated[key]) > lapse)
            except AttributeError:
                obj._cache_last_updated = defaultdict(lambda: 0)
                obj._cache_value = dict()
                update = True
            if update:
                obj._cache_value[key] = fn(obj, *args)
                obj._cache_last_updated[key] = now
            # gc3libs.log.debug("%s(%s, ...): Using cached value '%s'",
            #                  fn.__name__, obj, obj._cache_value[key])
            return obj._cache_value[key]
        return wrapper
    return decorator


def cat(*args, **extra_args):
    """
    Concatenate the contents of all `args` into `output`.  Both
    `output` and each of the `args` can be a file-like object or a
    string (indicating the path of a file to open).

    If `append` is `True`, then `output` is opened in append-only
    mode; otherwise it is overwritten.
    """
    output = extra_args.get('output', sys.stdout)
    append = extra_args.get('append', True)
    # ensure `output` is a file-like object, opened in write-mode
    try:
        output.write('')
    except:
        output = open(output, ifelse(append==True, 'a', 'w'))
    for arg in args:
        # ensure `arg` is a file-like object, opened in read-mode
        try:
            arg.read(0)
        except:
            arg = open(arg, 'r')
        for line in arg:
            output.write(line)


def copyfile(src, dst, overwrite=False, changed_only=True, link=False):
    """
    Copy a file from `src` to `dst`; return `True` if the copy was
    actually made.

    If `overwrite` is ``False`` (default), an existing destination entry
    is left unchanged and `False` is returned.

    If `overwrite` is ``True``, then `changed_only` determines
    if the destination file is overwritten:

    - if `changed_only` is ``True`` (default), then destination is
      overwritten if and only if it has a different size or has been
      modified less recently than the source;

    - if `changed_only` is ``False``, then the destination is
      overwritten unconditionally.

    If `link` is `True`, an attempt at hard-linking is done first;
    failing that, we copy the source file onto the destination
    one. Permission bits and modification times are copied as well.

    If `dst` is a directory, a file with the same basename as `src` is
    created (or overwritten) in the directory specified.

    Return ``True`` or ``False``, depending on whether the source file
    was actually copied (or linked) to the destination.
    """
    if os.path.isdir(dst):
        dst = os.path.join(dst, os.path.basename(src))
    if os.path.exists(dst) and not overwrite:
        return False
    if samefile(src, dst):
        return False
    try:
        if not os.path.exists(dst):
            dstdir = dirname(dst)
            if not os.path.exists(dstdir):
                os.makedirs(dstdir)
        else:
            # `dst` exists, check for changes
            if changed_only:
                sstat = os.stat(src)
                dstat = os.stat(dst)
                if (sstat.st_size == dstat.st_size and sstat.st_mtime <= dstat.st_mtime):
                    # same size and destination more recent, do not copy
                    return False
        if link:
            try:
                os.link(src, dst)
            except OSError, ex:
                # retry with normal copy
                shutil.copy2(src, dst)
        else:
            shutil.copy2(src, dst)
    except WindowsError:
        pass
    return True


def copytree(src, dst, overwrite=False, changed_only=True):
    """
    Recursively copy an entire directory tree rooted at `src`.

    If `overwrite` is ``False`` (default), entries that already exist in
    the destination tree are left unchanged and not overwritten.

    If `overwrite` is ``True``, then `changed_only` determines
    which files are overwritten:

    - if `changed_only` is ``True`` (default), then only files for
      which the source has a different size or has been modified
      more recently than the destination are copied;

    - if `changed_only` is ``False``, then *all* files in `source`
      will be copied into `destination`, unconditionally.

    Destination directory `dst` is created if it does not exist.

    See also: `shutil.copytree`.
    """
    assert os.path.isdir(src), \
        ("Source path `%s` does not name an existing directory" % src)
    errors = []
    if not os.path.exists(dst):
        os.makedirs(dst)
    for name in os.listdir(src):
        srcname = os.path.join(src, name)
        dstname = os.path.join(dst, name)
        try:
            if os.path.isdir(srcname):
                errors.extend(copytree(srcname, dstname, overwrite, changed_only))
            else:
                copyfile(srcname, dstname)
        except (IOError, os.error), why:
            errors.append((srcname, dstname, why))
    return errors


def copy_recursively(src, dst, overwrite=False, changed_only=True):
    """
    Copy `src` to `dst`, descending it recursively if necessary.

    The `overwrite` and `changed_only` optional arguments
    have the same effect as in `copytree`:func: (which see).
    """
    if os.path.isdir(src):
        copytree(src, dst, overwrite, changed_only)
    else:
        copyfile(src, dst, overwrite, changed_only)


def count(seq, predicate):
    """
    Return number of items in `seq` that match `predicate`.
    Argument `predicate` should be a callable that accepts
    one argument and returns a boolean.
    """
    count = 0
    for item in seq:
        if predicate(item):
            count += 1
    return count


def defproperty(fn):
    """
    Decorator to define properties with a simplified syntax in Python 2.4.
    See http://code.activestate.com/recipes/410698-property-decorator-for-python-24/#c6
    for details and examples.
    """
    p = { 'doc':fn.__doc__ }
    p.update(fn())
    return property(**p)


def deploy_configuration_file(filename, template_filename=None):
    """
    Ensure that configuration file `filename` exists; possibly
    copying it from the specified `template_filename`.

    Return `True` if a file with the specified name exists in the
    configuration directory.  If not, try to copy the template file
    over and then return `False`; in case the copy operations fails,
    a `NoConfigurationFile` exception is raised.

    The `template_filename` is always resolved relative to GC3Libs'
    'package resource' directory (i.e., the ``etc/`` directory in the
    sources.  If `template_filename` is `None`, then it is assumed to
    be the base name of `filename`.
    """
    if template_filename is None:
        template_filename = os.path.basename(filename)
    if not os.path.isabs(filename):
        filename = os.path.abspath(filename)
    if os.path.exists(filename):
        return True
    else:
        try:
            from pkg_resources import Requirement, resource_filename, DistributionNotFound
            import shutil
        except ImportError, err:
            raise gc3libs.exceptions.FatalError(
                "Cannot import required Python modules: %s."
                " Please check GC3Pie installation instructions at"
                " http://gc3pie.googlecode.com/svn/trunk/gc3pie/docs/html/install.html"
                % str(err))
        try:
            # copy sample config file
            if not os.path.exists(dirname(filename)):
                os.makedirs(dirname(filename))
            sample_config = resource_filename(Requirement.parse("gc3pie"),
                                              "gc3libs/etc/" + template_filename)
            shutil.copyfile(sample_config, filename)
            return False
        except IOError, err:
            gc3libs.log.critical("Failed copying configuration file: %s", err)
            raise gc3libs.exceptions.NoConfigurationFile("No configuration file '%s' was found, and an attempt to create it failed. Aborting." % filename)
        except DistributionNotFound, ex:
            raise AssertionError("Cannot access resources for Python package: %s."
                                 " Installation error?" % str(ex))


def dirname(pathname):
    """
    Same as `os.path.dirname` but return `.` in case of path names with no directory component.
    """
    # FIXME: figure out if this is a desirable outcome.  i.e. do we
    # want dirname to be empty, or do a pwd and find out what the
    # current dir is, or keep the "./".  I suppose this could make a
    # difference to some of the behavior of the scripts, such as
    # copying files around and such.
    return os.path.dirname(pathname) or '.'


class Enum(frozenset):
    """
    A generic enumeration class.  Inspired by:
    http://stackoverflow.com/questions/36932/whats-the-best-way-to-implement-an-enum-in-python/2182437#2182437
    with some more syntactic sugar added.

    An `Enum` class must be instanciated with a list of strings, that
    make the enumeration "label"::

      >>> Animal = Enum('CAT', 'DOG')

    Each label is available as an instance attribute, evaluating to
    itself::

      >>> Animal.DOG
      'DOG'

      >>> Animal.CAT == 'CAT'
      True

    As a consequence, you can test for presence of an enumeration
    label by string value::

      >>> 'DOG' in Animal
      True

    Finally, enumeration labels can also be iterated upon::

      >>> for a in sorted(Animal): print a
      CAT
      DOG
    """
    def __new__(cls, *args):
        return frozenset.__new__(cls, args)
    def __getattr__(self, name):
            if name in self:
                    return name
            else:
                    raise AttributeError("No '%s' in enumeration '%s'"
                                         % (name, self.__class__.__name__))
    def __setattr__(self, name, value):
            raise SyntaxError("Cannot assign enumeration values.")
    def __delattr__(self, name):
            raise SyntaxError("Cannot delete enumeration values.")


class ExponentialBackoff(object):
    """Generate waiting times with the `exponential backoff`_ algorithm.

    Returned times are in seconds (or fractions thereof); they are
    integral multiples of the basic time slot, which is set with the
    `slot_duration` constructor parameter.

    After `max_retries` have been attempted, any call to this iterator
    will raise a `StopIteration` exception.

    The `ExponentialBackoff` class implements the iterator protocol,
    so you can just retrieve waiting times with the `.next()` method,
    or by looping over it::

      >>> random.seed(314) # not-so-random for testing purposes...
      >>> for wt in ExponentialBackoff():
      ...   print wt,
      ...
      0.0 0.0 0.0 0.25 0.15 0.3

    .. _`exponential backoff`: http://en.wikipedia.org/wiki/Exponential_backoff#An_example_of_an_exponential_backoff_algorithm

    """

    def __init__(self, slot_duration=0.05, max_retries=5):
        self.attempt = -1
        self.slot_duration = slot_duration
        self.max_retries = max_retries

    def __iter__(self):
        return self

    def next(self):
        """Return next waiting time."""
        self.attempt += 1
        if self.attempt > self.max_retries:
            raise StopIteration
        return self.slot_duration * random.randint(0, 2**self.attempt - 1)

    def wait(self):
        """Wait for another while."""
        time.sleep(self.next())


def first(seq):
    """
    Return the first element of sequence or iterator `seq`.
    Raise `TypeError` if the argument does not implement
    either of the two interfaces.

    Examples::

      >>> s = [0, 1, 2]
      >>> first(s)
      0

      >>> s = {'a':1, 'b':2, 'c':3}
      >>> first(sorted(s.keys()))
      'a'
    """
    try: # try iterator interface
        return seq.next()
    except AttributeError:
        pass
    try: # seq is no iterator, try indexed lookup
        return seq[0]
    except IndexError:
        pass
    raise TypeError("Argument to `first()` method needs to be iterator or sequence.")

def fgrep(literal, filename):
    """
    Iterate over all lines in a file that contain the `literal` string.
    """
    with open(filename, 'r') as file:
        for line in file:
            if literal in line:
                yield line

def from_template(template, **extra_args):
    """
    Return the contents of `template`, substituting all occurrences
    of Python formatting directives '%(key)s' with the corresponding values
    taken from dictionary `extra_args`.

    If `template` is an object providing a `read()` method, that is
    used to gather the template contents; else, if a file named
    `template` exists, the template contents are read from it;
    otherwise, `template` is treated like a string providing the
    template contents itself.
    """
    if hasattr(template, 'read') and callable(template.read):
        template_contents = template.read()
    elif os.path.exists(template):
        template_file = file(template, 'r')
        template_contents = template_file.read()
        template_file.close()
    else:
        # treat `template` as a string
        template_contents = template
    # substitute `extra_args` into `t` and return it
    return (template_contents % extra_args)


def getattr_nested(obj, name):
    """
    Like Python's `getattr`, but perform a recursive lookup if `name` contains any dots.
    """
    dots = name.count('.')
    if dots == 0:
        return getattr(obj, name)
    else:
        first, rest = name.split('.', 1)
        return getattr(getattr(obj, first), rest)


def ifelse(test, if_true, if_false):
    """
    Return `if_true` is argument `test` evaluates to `True`,
    return `if_false` otherwise.

    This is just a workaround for Python 2.4 lack of the
    conditional assignment operator::

      >>> a = 1
      >>> b = ifelse(a, "yes", "no"); print b
      yes
      >>> b = ifelse(not a, 'yay', 'nope'); print b
      nope

    """
    if test:
        return if_true
    else:
        return if_false


# original source: https://gist.github.com/jtriley/7270594
def insert_char_every_n_chars(string, char='\n', every=64):
    return char.join(
        string[i:i + every] for i in xrange(0, len(string), every))


def irange(start, stop, step=1):
    """
    Iterate over all values greater or equal than `start` and less than `stop`.
    (Or the reverse, if `step < 0`.)

    Example::

      >>> list(irange(1, 5))
      [1, 2, 3, 4]
      >>> list(irange(0, 8, 3))
      [0, 3, 6]
      >>> list(irange(8, 0, -2))
      [8, 6, 4, 2]

    Unlike the built-in `range` function, `irange` also accepts
    floating-point values::

      >>> list(irange(0.0, 1.0, 0.5))
      [0.0, 0.5]

    Also unlike the built-in `range`, *both* `start` and `stop` have
    to be specified::

      >>> irange(42)
      Traceback (most recent call last):
        ...
      TypeError: irange() takes at least 2 arguments (1 given)

    Of course, a null `step` is not allowed::

      >>> list(irange(1, 2, 0))
      Traceback (most recent call last):
        ...
      AssertionError: Null step in irange.

    """
    assert float(step) != 0.0, "Null step in irange."
    value = start
    if step > 0:
        while value < stop:
            yield value
            value += step
    else: # step < 0
        while value > stop:
            yield value
            value += step


def lock(path, timeout, create=True):
    """
    Lock the file at `path`.  Raise a `LockTimeout` error if the lock
    cannot be acquired within `timeout` seconds.

    Return a `lock` object that should be passed unchanged to the
    `gc3libs.utils.unlock` function.

    If no `path` points to a non-existent location, an empty file is
    created before attempting to lock (unless `create` is `False`).
    An attempt is made to remove the file in case an error happens.

    See also: `gc3libs.utils.unlock`:func:
    """
    # ``FileLock`` requires that the to-be-locked file exists; if it
    # does not, we create an empty one (and avoid overwriting any
    # content, in case another process is also writing to it).  There
    # is thus no race condition here, as we attempt to lock the file
    # anyway, and this will stop concurrent processes.
    if not os.path.exists(path) and create:
        open(path, "a").close()
        created = True
    else:
        created = False
    try:
        lck = lockfile.FileLock(path, threaded=False)
        lck.acquire(timeout=timeout)
    except Exception, ex:
        if created:
            try:
                os.remove(path)
            except:
                pass
        raise
    return lck


class History(object):
    """
    A list of messages with timestamps and (optional) tags.

    The `append` method should be used to add a message to the `History`::

      >>> L = History()
      >>> L.append('first message')
      >>> L.append('second one')

    The `last` method returns the text of the last message appended, with its timestamp::

      >>> L.last().startswith('second one at')
      True

    Iterating over a `History` instance returns message texts in the
    temporal order they were added to the list, with their timestamp::

      >>> for msg in L: print(msg) # doctest: +ELLIPSIS
      first message ...

    """
    def __init__(self):
        self._messages = [ ]

    def append(self, message, *tags):
        """
        Append a message to this `History`.

        The message is timestamped with the time at the moment of the
        call.

        The optional `tags` argument is a sequence of strings. Tags
        are recorded together with the message and may be used to
        filter log messages given a set of labels. *(This feature is
        not yet implemented.)*

        """
        self._messages.append((message, time.time(), tags))

    def last(self):
        """
        Return text of last message appended.
        If log is empty, return empty string.
        """
        if len(self._messages) == 0:
            return ''
        else:
            return self.format_message(self._messages[-1])

    def format_message(self, message):
        """
        Return a formatted message, appending to the message its timestamp in human readable format.
        """
        return "%s at %s" % (message[0], time.asctime(time.localtime(message[1])))

    # shortcut for append
    def __call__(self, message, *tags):
        """Shortcut for `History.append` (which see)."""
        self.append(message, *tags)

    def __iter__(self):
        """Iterate over messages in the temporal order they were added."""
        return iter([self.format_message(record) for record in self._messages])

    def __str__(self):
        """Return all messages texts in a single string, separated by newline characters."""
        return str.join('\n', [self.format_message(record) for record in self._messages])


def mkdir(path, mode=0777):
    """
    Like `os.makedirs`, but does not throw an exception if PATH
    already exists.
    """
    if not os.path.exists(path):
        os.makedirs(path, mode)


def mkdir_with_backup(path, mode=0777):
    """
    Like `os.makedirs`, but if `path` already exists and is not empty,
    rename the existing one to a backup name (see the `backup` function).

    Unlike `os.makedirs`, no exception is thrown if the directory
    already exists and is empty, but the target directory permissions
    are not altered to reflect `mode`.
    """
    if os.path.isdir(path):
        if len(os.listdir(path)) > 0:
            # directory already exists and is non-empty; backup it and
            # make a new one
            backup(path)
            os.makedirs(path, mode)
        else:
            # keep existing empty directory
            pass
    else:
        os.makedirs(path, mode)


def movefile(src, dst, overwrite=False, changed_only=True, link=False):
    """
    Move a file from `src` to `dst`; return `True` if the move was
    actually made.

    The `overwrite` and `changed_only` optional arguments
    have the same effect as in `copyfile`:func: (which see).

    If `dst` is a directory, a file with the same basename as `src` is
    created (or overwritten) in the directory specified.

    Return ``True`` or ``False``, depending on whether the source file
    was actually moved to the destination.

    See also: `copyfile`:func:
    """
    if os.path.isdir(dst):
        dst = os.path.join(dst, os.path.basename(src))
    if os.path.exists(dst) and not overwrite:
        return False
    if samefile(src, dst):
        return False
    if not os.path.exists(dst):
        dstdir = dirname(dst)
        if not os.path.exists(dstdir):
            os.makedirs(dstdir)
    else:
        # `dst` exists, check for changes
        if changed_only:
            sstat = os.stat(src)
            dstat = os.stat(dst)
            if (sstat.st_size == dstat.st_size and sstat.st_mtime <= dstat.st_mtime):
                # same size and destination more recent, do not move
                return False
    try:
        shutil.move(src, dst)
    except WindowsError:
        pass
    return True


def movetree(src, dst, overwrite=False, changed_only=True):
    """
    Recursively move an entire directory tree rooted at `src`.

    The `overwrite` and `changed_only` optional arguments
    have the same effect as in `copytree`:func: (which see).

    See also: `copytree`:func:.
    """
    assert os.path.isdir(src), \
        ("Source path `%s` does not name an existing directory" % src)
    errors = []
    if not os.path.exists(dst):
        os.makedirs(dst)
    for name in os.listdir(src):
        srcname = os.path.join(src, name)
        dstname = os.path.join(dst, name)
        try:
            if os.path.isdir(srcname):
                errors.extend(movetree(srcname, dstname, overwrite, changed_only))
            else:
                movefile(srcname, dstname)
        except (IOError, os.error), why:
            errors.append((srcname, dstname, why))
    return errors


def move_recursively(src, dst, overwrite=False, changed_only=True):
    """
    Move `src` to `dst`, descending it recursively if necessary.

    The `overwrite` and `changed_only` optional arguments
    have the same effect as in `copytree`:func: (which see).
    """
    if os.path.isdir(src):
        movetree(src, dst, overwrite, changed_only)
    else:
        movefile(src, dst, overwrite, changed_only)


def prettyprint(D, indent=0, width=0, maxdepth=None, step=4,
                only_keys=None, output=sys.stdout, _key_prefix='', _exclude=None):
    """
    Print dictionary instance `D` in a YAML-like format.
    Each output line consists of:

      * `indent` spaces,
      * the key name,
      * a colon character ``:``,
      * the associated value.

    If the total line length exceeds `width`, the value is printed
    on the next line, indented by further `step` spaces; a value of 0 for
    `width` disables this line wrapping.

    Optional argument `only_keys` can be a callable that must return
    `True` when called with keys that should be printed, or a list of
    key names to print.

    Dictionary instances appearing as values are processed recursively
    (up to `maxdepth` nesting).  Each nested instance is printed
    indented `step` spaces from the enclosing dictionary.
    """
    # be sure we do not try to recursively dump `D`
    if _exclude is None:
        _exclude = set()
    _exclude.add(id(D))
    for k,v in D.iteritems():
        leading_spaces = indent * ' '
        full_name = "%s%s" % (_key_prefix, k)
        if only_keys is not None:
            try:
                # is `only_keys` a filter function?
                if not only_keys(str(full_name)):
                    continue
            except TypeError:
                # no, then it must be a list of key names, check for
                # keys having the same number of dots as in the prefix
                level = _key_prefix.count('.')
                found = False
                for name in only_keys:
                    # take only the initial segment, up to a "level" dots
                    dots = min(name.count('.'), level) + 1
                    prefix = str.join('.', name.split('.')[:dots])
                    if str(full_name) == prefix:
                        found = True
                        break
                if not found:
                    continue
        # ignore excluded items
        if id(v) in _exclude:
            continue
        first = str.join('', [leading_spaces, str(k), ': '])
        if isinstance(v, (dict, UserDict.DictMixin, UserDict.UserDict, OrderedDict)):
            if maxdepth is None or maxdepth > 0:
                if maxdepth is None:
                    depth = None
                else:
                    depth = maxdepth-1
                sstream = StringIO.StringIO()
                prettyprint(v, indent+step, width, depth, step,
                            only_keys, sstream, full_name+'.', _exclude)
                second = sstream.getvalue()
                sstream.close()
            elif maxdepth == 0:
                second = "..."
        elif isinstance(v, (list, tuple)):
            second = str.join(', ', [str(item) for item in v])
        else:
            second = str(v)
        # wrap overlong lines, and always wrap if the second part is multi-line
        if (width > 0 and len(first) + len(second) > width) or ('\n' in second):
            first += '\n'
        # indent a multi-line block by indent+step spaces
        if '\n' in second:
            lines = second.splitlines()
            # keep indentation relative to first line
            dedent = 0
            line0 = lines[0].expandtabs(step)
            while line0[dedent].isspace():
                dedent += 1
            # rebuild `second`, indenting each line by (indent+step) spaces
            second = ''
            for line in lines:
                second = str.join('', [
                    second,
                    ' ' * (indent+step),
                    line.rstrip().expandtabs(step)[dedent:],
                    '\n'
                    ])
        # there can be multiple trailing '\n's, which we remove here
        second = second.rstrip()
        # finally print line(s)
        output.write(first)
        output.write(second)
        output.write('\n')


def progressive_number(qty=None,
                       id_filename=os.path.expanduser('~/.gc3/next_id.txt')):
    """
    Return a positive integer, whose value is guaranteed to
    be monotonically increasing across different invocations
    of this function, and also across separate instances of the
    calling program.

    Example::

    (create a temporary directory to avoid bug #)

      >>> import tempfile, os
      >>> (fd, tmp) = tempfile.mkstemp()


      >>> n = progressive_number(id_filename=tmp)
      >>> m = progressive_number(id_filename=tmp)
      >>> m > n
      True

    If you specify a positive integer as argument, then a list of
    monotonically increasing numbers is returned.  For example::

      >>> ls = progressive_number(5, id_filename=tmp)
      >>> len(ls)
      5

    (clean up test environment)
      >>> os.remove(tmp)

    In other words, `progressive_number(N)` is equivalent to::

      nums = [ progressive_number() for n in range(N) ]

    only more efficient, because it has to obtain and release the lock
    only once.

    After every invocation of this function, the last returned number
    is stored into the file passed as argument `id_filename`.  If the
    file does not exist, an attempt to create it is made before
    allocating an id; the method can raise an `IOError` or `OSError`
    if `id_filename` cannot be opened for writing.

    *Note:* as file-level locking is used to serialize access to the
    counter file, this function may block (default timeout: 30
    seconds) while trying to acquire the lock, or raise a
    `LockTimeout` exception if this fails.

    :raise: LockTimeout, IOError, OSError

    :returns: A positive integer number, monotonically increasing with
            every call.  A list of such numbers if argument `qty` is a
            positive integer.
    """
    assert qty is None or qty > 0, \
        "Argument `qty` must be a positive integer"
    lck = lock(id_filename, timeout=30, create=True) # XXX: can raise 'LockTimeout'
    id_file = open(id_filename, 'r+')
    id = int(id_file.read(8) or "0", 16)
    id_file.seek(0)
    if qty is None:
        id_file.write("%08x -- DO NOT REMOVE OR ALTER THIS FILE: it is used internally by the gc3libs\n" % (id+1))
    else:
        id_file.write("%08x -- DO NOT REMOVE OR ALTER THIS FILE: it is used internally by the gc3libs\n" % (id+qty))
    id_file.close()
    unlock(lck)
    if qty is None:
        return id+1
    else:
        return [ (id+n) for n in range(1, qty+1) ]


def read_contents(path):
    """
    Return the whole contents of the file at `path` as a single string.

    Example::

      >>> read_contents('/dev/null')
      ''

      >>> import tempfile
      >>> (fd, tmpfile) = tempfile.mkstemp()
      >>> w = open(tmpfile, 'w')
      >>> w.write('hey')
      >>> w.close()
      >>> read_contents(tmpfile)
      'hey'

    (If you run this test, remember to do cleanup afterwards)
    ::

      >>> os.remove(tmpfile)

    """
    # XXX: this really calls for the `with:` statement...
    try:
        stream = open(path, 'r')
        return stream.read()
    finally:
        if 'stream' in locals():
            stream.close()


def safe_repr(obj):
    """
    Return a string describing Python object `obj`.

    Avoids calling any Python magic methods, so should be safe to use
    as a 'last resort' in implementation of `__str__` and `__repr__`.
    """
    return ("%s@%x" % (obj.__class__.__name__, id(obj)))


def same_docstring_as(referenced_fn):
    """
    Function decorator: sets the docstring of the following function
    to the one of `referenced_fn`.

    Intended usage is for setting docstrings on methods redefined in
    derived classes, so that they inherit the docstring from the
    corresponding abstract method in the base class.
    """
    def decorate(f):
            f.__doc__ = referenced_fn.__doc__
            return f
    return decorate


def samefile(path1, path2):
    """
    Like `os.path.samefile` but return `False` if either one of the
    paths does not exist.
    """
    try:
        return os.path.samefile(path1, path2)
    except OSError, err:
        if err.errno == 2: # ENOENT
            return False
        else:
            raise


def sh_quote_safe(text):
    """
    Escape a string for safely passing as argument to a shell command.

    Return a single-quoted string that expands to the exact literal
    contents of `text` when used as an argument to a shell command.
    Examples (note that backslashes are doubled because of Python's
    string read syntax)::

      >>> print(sh_quote_safe("arg"))
      'arg'
      >>> print(sh_quote_safe("'arg'"))
      ''\\''arg'\\'''

    """
    return ("'%s'" % text.replace("'", r"'\''"))


_DQUOTE_RE = re.compile(r'(\\*)"')
"""Regular expression for escaping double quotes in strings."""

def sh_quote_unsafe(text):
    """
    Double-quote a string for passing as argument to a shell command.

    Return a double-quoted string that expands to the contents of
    `text` but still allows variable expansion and ``\``-escapes
    processing by the UNIX shell.  Examples (note that backslashes are
    doubled because of Python's string read syntax)::

      >>> print(sh_quote_unsafe("arg"))
      "arg"
      >>> print(sh_quote_unsafe('"arg"'))
      "\\"arg\\""
      >>> print(sh_quote_unsafe(r'"\\"arg\\""'))
      "\\"\\\\\\"arg\\\\\\"\\""

    """
    return ('"%s"' % _DQUOTE_RE.sub(r'\1\1\"', text))


# see http://stackoverflow.com/questions/31875/is-there-a-simple-elegant-way-to-define-singletons-in-python/1810391#1810391
class Singleton(object):
    """
    Derived classes of `Singleton` can have only one instance in the
    running Python interpreter.

       >>> x = Singleton()
       >>> y = Singleton()
       >>> x is y
       True

    """
    def __new__(cls, *args, **extra_args):
        if not hasattr(cls, '_instance'):
            cls._instance = super(Singleton, cls).__new__(cls, *args, **extra_args)
        return cls._instance


class PlusInfinity(Singleton):
    """
    An object that is greater-than any other object.

        >>> x = PlusInfinity()

        >>> x > 1
        True
        >>> 1 < x
        True
        >>> 1245632479102509834570124871023487235987634518745 < x
        True

        >>> x > sys.maxint
        True
        >>> x < sys.maxint
        False
        >>> sys.maxint < x
        True

    `PlusInfinity` objects are actually larger than *any* given Python
    object::

        >>> x > 'azz'
        True
        >>> x > object()
        True

    Note that `PlusInfinity` is a singleton, therefore you always get
    the same instance when calling the class constructor::

        >>> x = PlusInfinity()
        >>> y = PlusInfinity()
        >>> x is y
        True

    Relational operators try to return the correct value when
    comparing `PlusInfinity` to itself::

        >>> x < y
        False
        >>> x <= y
        True
        >>> x == y
        True
        >>> x >= y
        True
        >>> x > y
        False

    """
    def __gt__(self, other):
        if self is other:
            return False
        else:
            return True
    def __ge__(self, other):
        return True
    def __lt__(self, other):
        return False
    def __le__(self, other):
        if self is other:
            return True
        else:
            return False
    def __eq__(self, other):
        if self is other:
            return True
        else:
            return False
    def __ne__(self, other):
        return not self.__eq__(other)


# In Python 2.7 still, `DictMixin` is an old-style class; thus, we need
# to make `Struct` inherit from `object` otherwise we loose properties
# when setting/pickling/unpickling
class Struct(object, UserDict.DictMixin):
    """
    A `dict`-like object, whose keys can be accessed with the usual
    '[...]' lookup syntax, or with the '.' get attribute syntax.

    Examples::

      >>> a = Struct()
      >>> a['x'] = 1
      >>> a.x
      1
      >>> a.y = 2
      >>> a['y']
      2

    Values can also be initially set by specifying them as keyword
    arguments to the constructor::

      >>> a = Struct(z=3)
      >>> a['z']
      3
      >>> a.z
      3

    Like `dict` instances, `Struct`s have a `copy` method to get a
    shallow copy of the instance:

      >>> b = a.copy()
      >>> b.z
      3

    """
    def __init__(self, initializer=None, **extra_args):
        if initializer is not None:
            try:
                # initializer is `dict`-like?
                for name, value in initializer.items():
                    self[name] = value
            except AttributeError:
                # initializer is a sequence of (name,value) pairs?
                for name, value in initializer:
                    self[name] = value
        for name, value in extra_args.items():
            self[name] = value

    def copy(self):
        """Return a (shallow) copy of this `Struct` instance."""
        return Struct(self)

    # the `DictMixin` class defines all std `dict` methods, provided
    # that `__getitem__`, `__setitem__` and `keys` are defined.
    def __setitem__(self, name, val):
        self.__dict__[name] = val
    def __getitem__(self, name):
        return self.__dict__[name]
    def keys(self):
        return self.__dict__.keys()


def string_to_boolean(word):
    """
    Convert `word` to a Python boolean value and return it.
    The strings `true`, `yes`, `on`, `1` (with any
    capitalization and any amount of leading and trailing
    spaces) are recognized as meaning Python `True`::

      >>> string_to_boolean('yes')
      True
      >>> string_to_boolean('Yes')
      True
      >>> string_to_boolean('YES')
      True
      >>> string_to_boolean(' 1 ')
      True
      >>> string_to_boolean('True')
      True
      >>> string_to_boolean('on')
      True

    Any other word is considered as boolean `False`::

      >>> string_to_boolean('no')
      False
      >>> string_to_boolean('No')
      False
      >>> string_to_boolean('Nay!')
      False
      >>> string_to_boolean('woo-hoo')
      False

    This includes also the empty string and whitespace-only::

      >>> string_to_boolean('')
      False
      >>> string_to_boolean('  ')
      False

    """
    if word.strip().lower() in [ 'true', 'yes', 'on', '1' ]:
        return True
    else:
        return False


def stripped(iterable):
    """
    Iterate over lines in `iterable` and return each of them stripped
    of leading and trailing blanks.
    """
    for item in iterable:
        yield item.strip()


@contextlib.contextmanager
def tempdir(**kwargs):
    """
    A context manager for creating and then deleting a temporary directory.

    All arguments are passed unchanged to the `tempfile.mkdtemp`
    standand library function.

    (Original source and credits: http://stackoverflow.com/a/10965572/459543)
    """
    tmpdir = tempfile.mkdtemp(**kwargs)
    try:
        yield tmpdir
    finally:
        if os.path.isdir(tmpdir):
            shutil.rmtree(tmpdir, ignore_errors=True)


def test_file(path, mode, exception=RuntimeError, isdir=False):
    """
    Test for access to a path; if access is not granted, raise an
    instance of `exception` with an appropriate error message.
    This is a frontend to `os.access`:func:, which see for exact
    semantics and the meaning of `path` and `mode`.

    :param path: Filesystem path to test.
    :param mode: See `os.access`:func:
    :param exception: Class of exception to raise if test fails.
    :param isdir: If `True` then also test that `path` points to a directory.

    If the test succeeds, `True` is returned::

      >>> test_file('/bin/cat', os.F_OK)
      True
      >>> test_file('/bin/cat', os.R_OK)
      True
      >>> test_file('/bin/cat', os.X_OK)
      True
      >>> test_file('/tmp', os.X_OK)
      True

    However, if the test fails, then an exception is raised::

      >>> test_file('/bin/cat', os.W_OK)
      Traceback (most recent call last):
        ...
      RuntimeError: Cannot write to file '/bin/cat'.

    If the optional argument `isdir` is `True`, then additionally test
    that `path` points to a directory inode::

      >>> test_file('/tmp', os.F_OK, isdir=True)
      True

      >>> test_file('/bin/cat', os.F_OK, isdir=True)
      Traceback (most recent call last):
        ...
      RuntimeError: Expected '/bin/cat' to be a directory, but it's not.
    """
    if not os.access(path, os.F_OK):
        raise exception("Cannot access %s '%s'."
                        % (ifelse(isdir, "directory", "file"), path))
    if isdir and not os.path.isdir(path):
        raise exception("Expected '%s' to be a directory, but it's not." % path)
    if (mode & os.R_OK) and not os.access(path, os.R_OK):
        raise exception("Cannot read %s '%s'."
                        % (ifelse(isdir, "directory", "file"), path))
    if (mode & os.W_OK) and not os.access(path, os.W_OK):
        raise exception("Cannot write to %s '%s'."
                        % (ifelse(isdir, "directory", "file"), path))
    if (mode & os.X_OK) and not os.access(path, os.X_OK):
        if isdir:
            raise exception("Cannot traverse directory '%s':"
                            " lacks 'x' permission." % path)
        else:
            raise exception("File '%s' lacks execute ('x') permission." % path)
    return True


def to_bytes(s):
    """
    Convert string `s` to an integer number of bytes.  Suffixes like
    'KB', 'MB', 'GB' (up to 'YB'), with or without the trailing 'B',
    are allowed and properly accounted for.  Case is ignored in
    suffixes.

    Examples::

      >>> to_bytes('12')
      12
      >>> to_bytes('12B')
      12
      >>> to_bytes('12KB')
      12000
      >>> to_bytes('1G')
      1000000000

    Binary units 'KiB', 'MiB' etc. are also accepted:

      >>> to_bytes('1KiB')
      1024
      >>> to_bytes('1MiB')
      1048576

    """
    last = -1
    unit = s[last].lower()
    if unit.isdigit():
        # `s` is a integral number
        return int(s)
    if unit == 'b':
        # ignore the the 'b' or 'B' suffix
        last -= 1
        unit = s[last].lower()
    if unit == 'i':
        k = 1024
        last -= 1
        unit = s[last].lower()
    else:
        k = 1000
    # convert the substring of `s` that does not include the suffix
    if unit.isdigit():
        return int(s[0:(last+1)])
    if unit == 'k':
        return int(float(s[0:last])*k)
    if unit == 'm':
        return int(float(s[0:last])*k*k)
    if unit == 'g':
        return int(float(s[0:last])*k*k*k)
    if unit == 't':
        return int(float(s[0:last])*k*k*k*k)
    if unit == 'p':
        return int(float(s[0:last])*k*k*k*k*k)
    if unit == 'e':
        return int(float(s[0:last])*k*k*k*k*k*k)
    if unit == 'z':
        return int(float(s[0:last])*k*k*k*k*k*k*k)
    if unit == 'y':
        return int(float(s[0:last])*k*k*k*k*k*k*k*k)


def send_mail(send_from, send_to, subject, text, files=[], server="localhost"):

    from smtplib import SMTP
    from email.MIMEMultipart import MIMEMultipart
    from email.MIMEBase import MIMEBase
    from email.MIMEText import MIMEText
    from email.Utils import COMMASPACE, formatdate
    from email import Encoders

    msg = MIMEMultipart()
    msg['From'] = send_from
    msg['To'] = COMMASPACE.join(send_to)
    msg['Date'] = formatdate(localtime=True)
    msg['Subject'] = subject

    msg.attach(MIMEText(text))

    for f in files:
        part = MIMEBase('application', "octet-stream")
        part.set_payload( open(f,"rb").read() )
        Encoders.encode_base64(part)
        part.add_header('Content-Disposition',
                        'attachment; filename="%s"' % os.path.basename(f))
        msg.attach(part)

    smtp = SMTP(server)
    smtp.sendmail(send_from, send_to, msg.as_string())
    smtp.close()


def touch(path):
    """
    Ensure a regular file exists at `path`.

    If the file already exists, its access and modification time are
    updated.

    (This is a very limited and stripped down version of the ``touch``
    POSIX utility.)
    """
    fd = open(path, 'a')
    fd.close()


def uniq(seq):
    """
    Iterate over all unique elements in sequence `seq`.

    Distinct values are returned in a sorted fashion.

    Examples:

      >>> for value in uniq([4,1,1,2,3,1,2]): print value
      ...
      1
      2
      3
      4

      >>> for value in uniq([1,2,3,4]): print value
      ...
      1
      2
      3
      4

      >>> for value in uniq([1,1,1,1]): print value
      ...
      1

    """
    return sorted(set(seq))


def unlock(lock):
    """
    Release a previously-acquired lock.

    Argument `lock` should be the return value of a previous
    `gc3libs.utils.lock` call.

    See also: `gc3libs.utils.lock`:func:
    """
    lock.release()


def update_parameter_in_file(path, var_in, new_val, regex_in):
    '''
    Updates a parameter value in a parameter file using predefined regular
    expressions in `_loop_regexps`.

    :param path: Full path to the parameter file.
    :param var_in: The variable to modify.
    :param new_val: The updated parameter value.
    :param regex: Name of the regular expression that describes the format of the parameter file.
    '''
    _loop_regexps = {
        'bar-separated':(r'([a-z]+[\s\|]+)'
                         r'(\w+)' # variable name
                         r'(\s*[\|]+\s*)' # bars and spaces
                         r'([\w\s\.,;\[\]\-]+)' # value
                         r'(\s*)'),
        'space-separated':(r'(\s*)'
                           r'(\w+)' # variable name
                           r'(\s+)' # spaces (filler)
                           r'([\w\s\.,;\[\]\-]+)' # values
                           r'(\s*)'), # spaces (filler)
    }
    isfound = False
    if regex_in in _loop_regexps.keys():
        regex_in = _loop_regexps[regex_in]
    para_file_in = open(path, 'r')
    para_file_out = open(path + '.tmp', 'w')
    for line in para_file_in:
        if not line.rstrip(): continue
        (a, var, b, old_val, c) = re.match(regex_in, line.rstrip()).groups()
        gc3libs.log.debug("Read variable '%s' with value '%s' ...", var, old_val)
        if var == var_in:
            isfound = True
            upd_val = new_val
        else:
            upd_val = old_val
        para_file_out.write(a + var + b + upd_val + c + '\n')
    para_file_out.close()
    para_file_in.close()
    # move new modified content over the old
    os.rename(path + '.tmp', path)
    if not isfound:
        gc3libs.log.critical('update_parameter_in_file could not find parameter in sepcified file')


def write_contents(path, data):
    """
    Overwrite the contents of the file at `path` with the given data.
    If the file does not exist, it is created.

    Example::

      >>> import tempfile
      >>> (fd, tmpfile) = tempfile.mkstemp()
      >>> write_contents(tmpfile, 'big data here')
      >>> read_contents(tmpfile)
      'big data here'

    (If you run this test, remember to clean up afterwards)
    ::

      >>> os.remove(tmpfile)

    """
    with open(path, 'wb') as stream:
        return stream.write(data)


class YieldAtNext(object):
    """
    Provide an alternate protocol for generators.

    Wrap a Python generator object, and buffer the return values from
    `send` and `throw` calls, returning `None` instead. Return the
    yielded value --or raise the `StopIteration` exception-- upon the
    subsequent call to the `next` method.
    """

    __slots__ = [ '_generator', '_has_saved', '_saved', '_stop_iteration' ]

    def __init__(self, generator):
        self._generator = generator
        self._has_saved = False
        self._stop_iteration = False

    def __iter__(self):
        return self

    def next(self):
        if self._stop_iteration:
            raise StopIteration
        elif self._has_saved:
            self._has_saved = False
            # XXX: This keeps a reference to the returned object into
            # `self._saved` until that attribute is overwritten.
            return self._saved
        else:
            return self._generator.next()

    def send(self, value):
        try:
            self._saved = self._generator.send(value)
            self._has_saved = True
        except StopIteration:
            self._stop_iteration = True

    def throw(self, *excinfo):
        try:
            self._saved = self._generator.throw(*excinfo)
            self._has_saved = True
        except StopIteration:
            self._stop_iteration = True


##
## Main
##

if __name__ == '__main__':
    import doctest
    doctest.testmod(name='utils',
                    optionflags=doctest.NORMALIZE_WHITESPACE)



#! /usr/bin/env python
#
"""
Unit tests for the `gc3libs.backends.shellcmd` module.
"""
# Copyright (C) 2011-2013 GC3, University of Zurich. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
__docformat__ = 'reStructuredText'
__version__ = '$Revision$'


import errno
import os
import shutil
import sys
import tempfile
import time

from nose.tools import raises, assert_equal, assert_not_equal

import gc3libs
from gc3libs.authentication import Auth
import gc3libs.config
import gc3libs.core
from gc3libs.quantity import Memory


class TestBackendShellcmd(object):
    CONF = """
[resource/localhost_test]
type=shellcmd
transport=local
# time_cmd=/usr/bin/time
# Use unusual values so that we can easily spot if the `override` option works
max_cores=123
max_cores_per_job=123
max_memory_per_core=999
max_walltime=2
architecture=x64_64
auth=noauth
enabled=True
override=False
resourcedir=%s

[auth/noauth]
type=none
"""

    def setUp(self):
        (fd, cfgfile) = tempfile.mkstemp()
        f = os.fdopen(fd, 'w+')
        CONFIG = TestBackendShellcmd.CONF % (cfgfile + '.d')
        f.write(CONFIG)
        f.close()
        self.files_to_remove = [cfgfile, cfgfile + '.d']
        self.apps_to_kill = []

        self.cfg = gc3libs.config.Configuration()
        self.cfg.merge_file(cfgfile)

        self.core = gc3libs.core.Core(self.cfg)
        self.backend = self.core.get_backend('localhost_test')
        # Update resource status
        self.backend.get_resource_status()

    def cleanup_file(self, fname):
        self.files_to_remove.append(fname)

    def tearDown(self):
        for fname in self.files_to_remove:
            if os.path.isdir(fname):
                shutil.rmtree(fname)
            elif os.path.exists(fname):
                os.remove(fname)

        for app in self.apps_to_kill:
            try:
                self.core.kill(app)
            except:
                pass
            try:
                self.core.free(app)
            except:
                pass

    def test_submission_ok(self):
        """
        Test a successful submission cycle and the backends' resource
        book-keeping.
        """
        tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')

        app = gc3libs.Application(
            arguments=['/usr/bin/env'],
            inputs=[],
            outputs=[],
            output_dir=tmpdir,
            stdout="stdout.txt",
            stderr="stderr.txt",
            requested_cores=1, )
        self.core.submit(app)
        self.apps_to_kill.append(app)

        self.cleanup_file(tmpdir)
        self.cleanup_file(app.execution.lrms_execdir)

        # there's no SUBMITTED state here: jobs go immediately into
        # RUNNING state
        assert_equal(app.execution.state, gc3libs.Run.State.SUBMITTED)
        assert_equal(self.backend.free_slots,  122)
        assert_equal(self.backend.user_queued, 0)
        assert_equal(self.backend.user_run,    1)

        # wait until the test job is done, but timeout and raise an error
        # if it takes too much time...
        MAX_WAIT = 10  # seconds
        WAIT = 0.1  # seconds
        waited = 0
        while app.execution.state != gc3libs.Run.State.TERMINATING \
                and waited < MAX_WAIT:
            time.sleep(WAIT)
            waited += WAIT
            self.core.update_job_state(app)
        try:
            assert_equal(app.execution.state, gc3libs.Run.State.TERMINATING)
            assert_equal(self.backend.free_slots,  123)
            assert_equal(self.backend.user_queued, 0)
            assert_equal(self.backend.user_run,    0)
        except:
            self.core.fetch_output(app)
            self.core.free(app)
            raise

        self.core.fetch_output(app)
        try:
            assert_equal(app.execution.state, gc3libs.Run.State.TERMINATED)
            assert_equal(self.backend.free_slots,  123)
            assert_equal(self.backend.user_queued, 0)
            assert_equal(self.backend.user_run,    0)
        except:
            self.core.free(app)
            raise

    def test_check_app_after_reloading_session(self):
        """Check if we are able to check the status of a job after the
        script which started the job has died.
        """
        tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
        self.cleanup_file(tmpdir)

        app = gc3libs.Application(
            arguments=['/usr/bin/env'],
            inputs=[],
            outputs=[],
            output_dir=tmpdir,
            stdout="stdout.txt",
            stderr="stderr.txt",
            requested_cores=1, )
        self.core.submit(app)
        self.apps_to_kill.append(app)

        self.cleanup_file(app.execution.lrms_execdir)
        pid = app.execution.lrms_jobid

        # The wrapper process should die and write the final status
        # and the output to a file, so that `Core` will be able to
        # retrieve it.

        # wait until the test job is done, but timeout and raise an error
        # if it takes too much time...
        MAX_WAIT = 10  # seconds
        WAIT = 0.1  # seconds
        waited = 0
        while app.execution.state != gc3libs.Run.State.TERMINATING \
                and waited < MAX_WAIT:
            time.sleep(WAIT)
            waited += WAIT
            self.core.update_job_state(app)

        assert_equal(app.execution.state, gc3libs.Run.State.TERMINATING)
        assert_equal(app.execution.returncode, 0)

    def test_app_argument_with_spaces(self):
        """Check that arguments with spaces are not splitted
        """
        tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
        self.cleanup_file(tmpdir)

        app = gc3libs.Application(
            arguments=['/bin/ls', '-d', '/ /'],
            inputs=[],
            outputs=[],
            output_dir=tmpdir,
            stdout="stdout.txt",
            stderr="stderr.txt",
            requested_cores=1, )
        self.core.submit(app)
        self.apps_to_kill.append(app)

        self.cleanup_file(app.execution.lrms_execdir)
        MAX_WAIT = 10  # seconds
        WAIT = 0.1  # seconds
        waited = 0
        while app.execution.state != gc3libs.Run.State.TERMINATING \
                and waited < MAX_WAIT:
            time.sleep(WAIT)
            waited += WAIT
            self.core.update_job_state(app)
        assert_equal(app.execution.state, gc3libs.Run.State.TERMINATING)
        assert_not_equal(app.execution.returncode, 0)

    # def test_time_cmd_args(self):
    #     assert_equal(self.backend.time_cmd, '/usr/bin/time')

    def test_resource_usage(self):
        tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
        self.cleanup_file(tmpdir)

        app = gc3libs.Application(
            arguments=['/bin/echo', 'Hello', 'World'],
            inputs=[],
            outputs=[],
            output_dir=tmpdir,
            requested_cores=2,
            requested_memory=10 * Memory.MB, )
        cores_before = self.backend.free_slots
        mem_before = self.backend.available_memory
        self.core.submit(app)
        self.apps_to_kill.append(app)

        cores_after = self.backend.free_slots
        mem_after = self.backend.available_memory
        assert_equal(cores_before, cores_after + 2)
        assert_equal(mem_before, mem_after + app.requested_memory)
        MAX_WAIT = 10  # seconds
        WAIT = 0.1  # seconds
        waited = 0
        while app.execution.state != gc3libs.Run.State.TERMINATING \
                and waited < MAX_WAIT:
            time.sleep(WAIT)
            waited += WAIT
            self.core.update_job_state(app)
        assert_equal(self.backend.free_slots, cores_before)
        avail = self.backend.available_memory
        assert_equal(self.backend.available_memory, mem_before)

    @raises(gc3libs.exceptions.NoResources)
    def test_not_enough_cores_usage(self):
        tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
        self.cleanup_file(tmpdir)
        bigapp = gc3libs.Application(
            arguments=['/bin/echo', 'Hello', 'World'],
            inputs=[],
            outputs=[],
            output_dir=tmpdir,
            requested_cores=self.backend.free_slots + 1,
            requested_memory=10 * Memory.MiB, )
        self.core.submit(bigapp)

    @raises(gc3libs.exceptions.NoResources)
    def test_not_enough_memory_usage(self):
        tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
        self.cleanup_file(tmpdir)
        bigapp = gc3libs.Application(
            arguments=['/bin/echo', 'Hello', 'World'],
            inputs=[],
            outputs=[],
            output_dir=tmpdir,
            requested_cores=1,
            requested_memory=self.backend.total_memory + Memory.B, )
        self.core.submit(bigapp)


class TestBackendShellcmdCFG(object):
    CONF = """
[resource/localhost_test]
type=shellcmd
transport=local
time_cmd=/usr/bin/time
max_cores=1000
max_cores_per_job=4
max_memory_per_core=2
max_walltime=2
architecture=x64_64
auth=noauth
enabled=True
override=%s
resourcedir=%s

[auth/noauth]
type=none
"""

    def setUp(self):
        self.files_to_remove = []

    def cleanup_file(self, fname):
        self.files_to_remove.append(fname)

    def tearDown(self):
        for fname in self.files_to_remove:
            if os.path.isdir(fname):
                shutil.rmtree(fname)
            elif os.path.exists(fname):
                os.remove(fname)

    def test_override_cfg_flag(self):
        (fd, cfgfile) = tempfile.mkstemp()
        f = os.fdopen(fd, 'w+')
        f.write(TestBackendShellcmdCFG.CONF % ("True", cfgfile + '.d'))
        f.close()
        self.files_to_remove = [cfgfile, cfgfile + '.d']

        self.cfg = gc3libs.config.Configuration()
        self.cfg.merge_file(cfgfile)

        self.core = gc3libs.core.Core(self.cfg)
        self.backend = self.core.get_backend('localhost_test')
        # Update resource status
        self.backend.get_resource_status()

        assert self.backend.max_cores < 1000

    def test_do_not_override_cfg_flag(self):
        (fd, cfgfile) = tempfile.mkstemp()
        f = os.fdopen(fd, 'w+')
        f.write(TestBackendShellcmdCFG.CONF % ("False", cfgfile + '.d'))
        f.close()
        self.files_to_remove = [cfgfile, cfgfile + '.d']

        self.cfg = gc3libs.config.Configuration()
        self.cfg.merge_file(cfgfile)

        self.core = gc3libs.core.Core(self.cfg)
        self.backend = self.core.get_backend('localhost_test')
        # Update resource status

        assert_equal(self.backend.max_cores, 1000)

    def test_resource_sharing_w_multiple_backends(self):
        tmpdir = tempfile.mkdtemp(prefix=__name__, suffix='.d')
        (fd, cfgfile) = tempfile.mkstemp()
        f = os.fdopen(fd, 'w+')
        f.write(TestBackendShellcmdCFG.CONF % ("False", cfgfile + '.d'))
        f.close()
        self.files_to_remove = [cfgfile, cfgfile + '.d', tmpdir]

        cfg1 = gc3libs.config.Configuration()
        cfg1.merge_file(cfgfile)

        cfg2 = gc3libs.config.Configuration()
        cfg2.merge_file(cfgfile)

        core1 = gc3libs.core.Core(cfg1)
        core2 = gc3libs.core.Core(cfg2)

        backend1 = core1.get_backend('localhost_test')
        backend2 = core2.get_backend('localhost_test')

        app = gc3libs.Application(
            arguments=['/bin/echo', 'Hello', 'World'],
            inputs=[],
            outputs=[],
            output_dir=tmpdir,
            requested_cores=1,
            requested_memory=10 * Memory.MiB, )

        try:
            core1.submit(app)
            assert_equal(backend1.free_slots, backend1.max_cores - app.requested_cores)

            assert_equal(backend2.free_slots, backend2.max_cores)
            backend2.get_resource_status()
            assert_equal(backend2.free_slots, backend2.max_cores - app.requested_cores)
        finally:
            core1.kill(app)
            core1.free(app)


if __name__ == "__main__":
    import nose
    nose.runmodule()



#! /usr/bin/env python
#
"""
"""
# Copyright (C) 2011, 2012, GC3, University of Zurich. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
__docformat__ = 'reStructuredText'
__version__ = '$Revision$'


from nose.tools import raises
from nose.plugins.skip import SkipTest

from gc3libs import Application, Run, Task
from gc3libs.workflow import RetryableTask
import gc3libs.exceptions

class TestApplication(Application):

    def __init__(self):
        Application.__init__(self,
                             arguments = [''],
                             inputs = [],
                             outputs = [],
                             output_dir = None)
        self.execution.state = 'TERMINATED'
        self.execution.returncode = 0
        self.changed = False

    def update_state(self):
        # self.changed = False
        pass

@raises(AssertionError)
def test_persisted_change():
    app = TestApplication()
    task = RetryableTask(app)
    # task.execution.state = 'RUNNING'

    # This is supposed to alter the state of the task
    # thus mark it as 'changed'
    task.update_state()

    # We expect task.changed to be true
    assert(task.changed == False)

if "__main__" == __name__:
    import nose
    nose.runmodule()



#! /usr/bin/env python
#
#   gnw_measures.py -- Front-end script for evaluating R-based 'grid'
#   function over a large dataset.
#
#   Copyright (C) 2011, 2012 GC3, University of Zurich
#
#   This program is free software: you can redistribute it and/or
#   modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Front-end script for submitting multiple `R` jobs.
It uses the generic `gc3libs.cmdline.SessionBasedScript` framework.

See the output of ``gnw_measures.py --help`` for program usage
instructions.

Input parameters consists of:
:param str id_times file: Path to an .csv file containing input data.

Example: 
"id","ts_reference","tf_reference"
"12802",1663058794000,Inf
"2617",1666361937000,1666362018000

Optional Data file in .csv format.

Example:
"userid_hash","friendid_hash","t_friendcreated"
"15","33",1647193283000
"44","81",1659876972000


XXX: To be clarified:
. When input files should be removed ?
. What happen if an error happen at merging time ?
. Should be possible to re-run a subset of the initial chunk list
without re-creating a new session ?
e.g. adding a new argument accepting chunk ranges (-R 3000:7500)
This would trigger the re-run of the whole workflow only 
for lines between 3000 and 7500 
"""

__version__ = '2.2.1 version (SVN $Revision$)'
# summary of user-visible changes
__changelog__ = """
  2013-11-07:
  * Initial version
"""
__author__ = 'Sergio Maffioletti <sergio.maffioletti@gc3.uzh.ch>'
__docformat__ = 'reStructuredText'


# run script, but allow GC3Pie persistence module to access classes defined here;
# for details, see: http://code.google.com/p/gc3pie/issues/detail?id=95
if __name__ == "__main__":
    import gnw_measures
    gnw_measures.GnwScript().run()

import os
import sys
import time
import tempfile

import shutil

from pkg_resources import Requirement, resource_filename

import gc3libs
import gc3libs.exceptions
from gc3libs import Application, Run, Task
from gc3libs.cmdline import SessionBasedScript, executable_file
import gc3libs.utils
from gc3libs.quantity import Memory, kB, MB, GB, Duration, hours, minutes, seconds
from gc3libs.workflow import RetryableTask

class GnwApplication(Application):
    """
    Custom class to wrap the execution of the R function over 
    a chunked ``id_times`` file.
    """
    application_name = 'gnw_measures'
    
    def __init__(self, id_times_filename, **extra_args):

        # setup output references
        self.output_dir = extra_args['output_dir']
        self.output_filename = 'result-%s.csv' % extra_args['index_chunk']
        self.output_file = os.path.join(self.output_dir,self.output_filename)

        outputs = [("./result.csv",self.output_filename)]

        # setup input references
        inputs = dict()

        # Rscript --vanilla $MASTER $GRID $IDS $FRIENDSHIP
        arguments = "Rscript --vanilla "

        # Master driver script
        if extra_args.has_key('driver_script'):
            inputs[extra_args['driver_script']] = "./bin/run.R"
            arguments +=  "./bin/run.R "
        else:
            # Use default
            arguments +=  "~/bin/run.R "
            
        # Grid function
        if extra_args.has_key('grid_function'):
            inputs[extra_args['grid_function']] = "./bin/grid.r"
            arguments += "./bin/grid.r "
        else:
            # Use default
            arguments += "~/bin/grid.r "

        # ID times
        inputs[id_times_filename] = "./id_times.csv"
        arguments += "./id_times.csv "

        if extra_args.has_key('data'):
            inputs[extra_args['data']] = "./data/friendship_nw.csv"
            arguments += "./data/friendship_nw.csv "

        Application.__init__(
            self,
            arguments = arguments,
            inputs = inputs,
            outputs = outputs,
            stdout = 'gnw_measures.log',
            join=True,
            **extra_args)

    def terminated(self):
        """
        Check whether output file has been properly created
        """
        # XXX: TBD, work on a more precise checking of the output file
        gc3libs.log.info("Application terminated with exit code %s" % self.execution.exitcode)
        if (not os.path.isfile(self.output_file)):
            gc3libs.log.error("Failed while checking outputfile %s." % self.output_file)
            # Retry
            self.execution.returncode = (0, 99)


class GnwTask(RetryableTask):
    def __init__(self, id_times_filename, **extra_args):
        RetryableTask.__init__(
            self,
            # actual computational job
            GnwApplication(
                id_times_filename, 
                **extra_args),
            **extra_args
            )

class GnwScript(SessionBasedScript):
    """
    Splits input .csv file into smaller chunks, each of them of size 
    'self.params.chunk_size'.
    Then it submits one execution for each of the created chunked files.
    
    The ``gnw_measures`` command keeps a record of jobs (submitted, executed
    and pending) in a session file (set name with the ``-s`` option); at
    each invocation of the command, the status of all recorded jobs is
    updated, output from finished jobs is collected, and a summary table
    of all known jobs is printed.
    
    Options can specify a maximum number of jobs that should be in
    'SUBMITTED' or 'RUNNING' state; ``gnw_measures`` will delay submission of
    newly-created jobs so that this limit is never exceeded.

    Once the processing of all chunked files has been completed, ``gnw_measures``
    aggregates them into a single larger output file located in 
    'self.params.output'.
    """

    def __init__(self):
        SessionBasedScript.__init__(
            self,
            version = __version__, # module version == script version
            application = GnwTask, 
            # only display stats for the top-level policy objects
            # (which correspond to the processed files) omit counting
            # actual applications because their number varies over
            # time as checkpointing and re-submission takes place.
            stats_only_for = GnwTask,
            )

    def setup_options(self):
        self.add_param("-k", "--chunk", metavar="[NUM]", #type=executable_file,
                       dest="chunk_size", default="1000",
                       help="How to split the edges input data set.")
        
        self.add_param("-M", "--master", metavar="[PATH]",
                       dest="driver_script", default=None,
                       help="Location of master driver R script.")

        self.add_param("-D", "--data", metavar="[PATH]",
                       dest="data", default=None,
                       help="Location of the reference friendship date in .cvs format.")

        self.add_param("-F", "--grid", metavar="[PATH]",
                       dest="grid_function", default=None,
                       help="Location of the grid function R script.")

    def setup_args(self):

        self.add_param('id_times', type=str,
                       help="Input data full path name.")

    def parse_args(self):
        """
        Check presence of input folder (should contains R scripts).
        path to command_file should also be valid.
        """
        
        # check input arguments
        if not os.path.isfile(self.params.id_times):
            raise gc3libs.exceptions.InvalidUsage(
                "Invalid path to input data: '%s'. File not found"
                % self.params.id_times)

        self.id_times_filename = os.path.basename(self.params.id_times)

        # Verify that 'self.params.chunk_size' is int
        self.params.chunk_size = int(self.params.chunk_size)

    def new_tasks(self, extra):
        """
        Read content of 'command_file'
        For each command line, generate a new GcgpsTask
        """
        tasks = []

        for (input_file, index_chunk) in self._generate_chunked_files_and_list(self.params.id_times,
                                                                              self.params.chunk_size):            
            jobname = "gnw_measures-%s" % (str(index_chunk))

            extra_args = extra.copy()

            extra_args['index_chunk'] = str(index_chunk)

            extra_args['jobname'] = jobname
            
            extra_args['output_dir'] = self.params.output
            extra_args['output_dir'] = extra_args['output_dir'].replace('NAME', 
                                                                        os.path.join('computation',
                                                                                     jobname))
            extra_args['output_dir'] = extra_args['output_dir'].replace('SESSION', 
                                                                        os.path.join('computation',
                                                                                     jobname))
            extra_args['output_dir'] = extra_args['output_dir'].replace('DATE', 
                                                                        os.path.join('computation',
                                                                                     jobname))
            extra_args['output_dir'] = extra_args['output_dir'].replace('TIME', 
                                                                        os.path.join('computation',
                                                                                     jobname))
            

            if self.params.driver_script:
                extra_args['driver_script'] = self.params.driver_script
            if self.params.data:
                extra_args['data'] = self.params.data
            if self.params.grid_function:
                extra_args['grid_function'] = self.params.grid_function

            self.log.debug("Creating Task for index : %d - %d" %
                           (index_chunk, (index_chunk + self.params.chunk_size)))

            tasks.append(GnwTask(
                    input_file,
                    **extra_args))

        return tasks

    def after_main_loop(self):
        """
        Merge all result files together
        Then clean up tmp files
        """
        # init result .csv file
        merged_csv = "result-%s" % os.path.basename(self.params.id_times)

        try:
            fout=open(merged_csv,"w+")
            for task in self.session:
                if isinstance(task,GnwTask) and task.execution.returncode == 0:
                    try:
                        for line in open(task.output_file):
                            fout.write(line)    
                    except OSError, osx:
                        # Report failure and continue
                        # XXX: Check what would be the correct behaviour
                        gc3libs.log.error("Failed while merging result file %s." % task.output_file)
                        continue
            fout.close()
        except OSError, osx:
            gc3libs.log.critical("Failed while merging result files. " +
                                 "Error %s" % str(osx))
            raise
        finally:
            fout.close()

    def _generate_chunked_files_and_list(self, file_to_chunk, chunk_size=1000):
        """
        Takes a file_name as input and a defined chunk size
        ( uses 1000 as default )
        returns a list of filenames 1 for each chunk created and a corresponding
        index reference
        e.g. ('/tmp/chunk2800.csv,2800) for the chunk segment that goes
        from 2800 to (2800 + chunk_size)
        """
        index = 0
        chunk = []
        failure = False
        chunk_files_dir = os.path.join(self.session.path,"tmp")

        # creating 'chunk_files_dir'
        if not(os.path.isdir(chunk_files_dir)):
            try:
                os.mkdir(chunk_files_dir)
            except OSError, osx:
                gc3libs.log.error("Failed while creating tmp folder %s. " % chunk_files_dir +
                                  "Error %s." % str(osx) +
                                  "Using default '/tmp'")
                chunk_files_dir = "/tmp"

        try:
            fd = open(file_to_chunk,'rb')

            # Read header
            header = fd.readline()

            fout = None

            for (i, line) in enumerate(fd):
                if i % chunk_size == 0:
                    if fout: 
                        fout.close()
                    (handle, self.tmp_filename) = tempfile.mkstemp(dir=chunk_files_dir,
                                                                    prefix=
                                                                   'gnw_measuresn-', 
                                                                   suffix=
                                                                   "%d.csv" % i)
                    fout = open(self.tmp_filename,'w')
                    chunk.append((fout.name,i))
                    fout.write(header)
                fout.write(line)
            fout.close()
        except OSError, osx:
            gc3libs.log.critical("Failed while creating chunk files." +
                                 "Error %s", (str(osx)))
            failure = True
        finally:
            if failure:
                # remove all tmp file created
                gc3libs.log.info("Could not generate full chunk list. "
                                 "Removing all existing tmp files... ")
                for (cfile, index) in chunk:
                    try:
                        os.remove(cfile)
                    except OSError, osx:
                        gc3libs.log.error("Failed while removing " +
                                          "tmp file %s. " +
                                          "Message %s" % osx.message)

        return chunk
